@@ -1,9 +1,9 @@
 /*
- * "$Id: lpq.c,v 1.20 2002/01/02 17:58:35 mike Exp $"
+ * "$Id: lpq.c,v 1.32 2005/01/03 19:29:44 mike Exp $"
  *
  *   "lpq" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -26,6 +26,7 @@
  *   main()         - Parse options and commands.
  *   show_jobs()    - Show jobs.
  *   show_printer() - Show printer status.
+ *   usage()        - Show program usage.
  */
 
 /*
@@ -38,8 +39,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
-#include <config.h>
+#include <cups/string.h>
 #include <cups/cups.h>
 #include <cups/language.h>
 #include <cups/debug.h>
@@ -52,6 +52,7 @@
 static int	show_jobs(http_t *, const char *, const char *, const int,
 		          const int);
 static void	show_printer(http_t *, const char *);
+static void	usage(void);
 
 
 /*
@@ -65,16 +66,18 @@ main(int  argc,		/* I - Number of command-line arguments */
   int		i;		/* Looping var */
   http_t	*http;		/* Connection to server */
   const char	*dest,		/* Desired printer */
-		*user;		/* Desired user */
+		*user,		/* Desired user */
+		*val;		/* Environment variable name */
   char		*instance;	/* Printer instance */
   int		id,		/* Desired job ID */
+		all,		/* All printers */
 		interval,	/* Reporting interval */
 		longstatus;	/* Show file details */
   int		num_dests;	/* Number of destinations */
   cups_dest_t	*dests;		/* Destinations */
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
   http_encryption_t encryption;	/* Encryption? */
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 
 
  /*
@@ -97,13 +100,10 @@ main(int  argc,		/* I - Number of command-line arguments */
   id         = 0;
   interval   = 0;
   longstatus = 0;
+  all        = 0;
 
   num_dests = cupsGetDests(&dests);
 
-  for (i = 0; i < num_dests; i ++)
-    if (dests[i].is_default)
-      dest = dests[i].name;
-
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '+')
       interval = atoi(argv[i] + 1);
@@ -112,7 +112,7 @@ main(int  argc,		/* I - Number of command-line arguments */
       switch (argv[i][1])
       {
         case 'E' : /* Encrypt */
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
 	    encryption = HTTP_ENCRYPT_REQUIRED;
 
 	    if (http)
@@ -120,7 +120,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 #else
             fprintf(stderr, "%s: Sorry, no encryption support compiled in!\n",
 	            argv[0]);
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 	    break;
 
         case 'P' : /* Printer */
@@ -129,15 +129,35 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    else
 	    {
 	      i ++;
+
+	      if (i >= argc)
+	      {
+		httpClose(http);
+		cupsFreeDests(num_dests, dests);
+	        
+	        usage();
+	      }
+
 	      dest = argv[i];
 	    }
 
 	    if ((instance = strchr(dest, '/')) != NULL)
-	      *instance = '\0';
+	      *instance++ = '\0';
+
+            if (cupsGetDest(dest, instance, num_dests, dests) == NULL)
+	    {
+	      if (instance)
+		fprintf(stderr, "lpq: Unknown destination \"%s/%s\"!\n",
+		        dest, instance);
+              else
+		fprintf(stderr, "lpq: Unknown destination \"%s\"!\n", dest);
+
+	      return (1);
+	    }
 	    break;
 
 	case 'a' : /* All printers */
-	    dest = NULL;
+	    all = 1;
 	    break;
 
 	case 'l' : /* Long status */
@@ -145,17 +165,52 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    break;
 
 	default :
-	    fputs("Usage: lpq [-P dest] [-l] [+interval]\n", stderr);
 	    httpClose(http);
-            cupsFreeDests(num_dests, dests);
-	    return (1);
+	    cupsFreeDests(num_dests, dests);
+
+	    usage();
+	    break;
       }
     }
-    else if (isdigit(argv[i][0]))
+    else if (isdigit(argv[i][0] & 255))
       id = atoi(argv[i]);
     else
       user = argv[i];
 
+  if (dest == NULL && !all)
+  {
+    for (i = 0; i < num_dests; i ++)
+      if (dests[i].is_default)
+	dest = dests[i].name;
+
+    if (dest == NULL)
+    {
+      val = NULL;
+
+      if ((dest = getenv("LPDEST")) == NULL)
+      {
+	if ((dest = getenv("PRINTER")) != NULL)
+	{
+          if (!strcmp(dest, "lp"))
+            dest = NULL;
+	  else
+	    val = "PRINTER";
+	}
+      }
+      else
+	val = "LPDEST";
+
+      if (dest && !cupsGetDest(dest, NULL, num_dests, dests))
+	fprintf(stderr, "lp: error - %s environment variable names non-existent destination \"%s\"!\n",
+        	val, dest);
+      else
+	fputs("lpq: error - no default destination available.\n", stderr);
+      httpClose(http);
+      cupsFreeDests(num_dests, dests);
+      return (1);
+    }
+  }
+
  /*
   * Show the status in a loop...
   */
@@ -403,7 +458,16 @@ show_jobs(http_t     *http,	/* I - HTTP connection to server */
 	strcpy(rankstr, "active");
       else
       {
-	snprintf(rankstr, sizeof(rankstr), "%d%s", rank, ranks[rank % 10]);
+       /*
+        * Make the rank show the "correct" suffix for each number
+	* (11-13 are the only special cases, for English anyways...)
+	*/
+
+	if ((rank % 100) >= 11 && (rank % 100) <= 13)
+	  snprintf(rankstr, sizeof(rankstr), "%dth", rank);
+	else
+	  snprintf(rankstr, sizeof(rankstr), "%d%s", rank, ranks[rank % 10]);
+
 	rank ++;
       }
 
@@ -415,20 +479,17 @@ show_jobs(http_t     *http,	/* I - HTTP connection to server */
 	  snprintf(namestr, sizeof(namestr), "%d copies of %s", jobcopies,
 	           jobname);
 	else
-	{
-	  strncpy(namestr, jobname, sizeof(namestr) - 1);
-	  namestr[sizeof(namestr) - 1] = '\0';
-	}
+	  strlcpy(namestr, jobname, sizeof(namestr));
 
-        printf("%s: %-34.34s[job %d localhost]\n", jobuser, rankstr, jobid);
-        printf("        %-40.40s%d bytes\n", namestr, jobsize);
+        printf("%s: %-33.33s [job %d localhost]\n", jobuser, rankstr, jobid);
+        printf("        %-39.39s %d bytes\n", namestr, jobsize);
       }
       else
 #ifdef __osf__
         printf("%-6s %-10.10s %-4d %-10d %-27.27s %d bytes\n", rankstr, jobuser,
 	       jobpriority, jobid, jobname, jobsize);
 #else
-        printf("%-7s %-8.8s%-8d%-32.32s%d bytes\n", rankstr, jobuser,
+        printf("%-7s %-7.7s %-7d %-31.31s %d bytes\n", rankstr, jobuser,
 	       jobid, jobname, jobsize);
 #endif /* __osf */
 
@@ -538,5 +599,17 @@ show_printer(http_t     *http,	/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: lpq.c,v 1.20 2002/01/02 17:58:35 mike Exp $".
+ * 'usage()' - Show program usage.
+ */
+
+static void
+usage(void)
+{
+  fputs("Usage: lpq [-P dest] [-l] [+interval]\n", stderr);
+  exit(1);
+}
+
+
+/*
+ * End of "$Id: lpq.c,v 1.32 2005/01/03 19:29:44 mike Exp $".
  */
