@@ -1,9 +1,9 @@
 /*
- * "$Id: lpr.c,v 1.23 2002/01/02 17:58:35 mike Exp $"
+ * "$Id: lpr.c,v 1.36 2005/01/03 19:29:44 mike Exp $"
  *
  *   "lpr" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -33,12 +33,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
 
-#include <config.h>
+#include <cups/string.h>
 #include <cups/cups.h>
 
 
 #ifndef WIN32
+#  include <unistd.h>
 #  include <signal.h>
 
 
@@ -70,7 +72,8 @@ main(int  argc,		/* I - Number of command-line arguments */
   char		ch;		/* Option character */
   char		*printer,	/* Destination printer or class */
 		*instance;	/* Instance */
-  const char	*title;		/* Job title */
+  const char	*title,		/* Job title */
+		*val;		/* Environment variable name */
   int		num_copies;	/* Number of copies per file */
   int		num_files;	/* Number of files to print */
   const char	*files[1000];	/* Files to print */
@@ -84,6 +87,7 @@ main(int  argc,		/* I - Number of command-line arguments */
   int		temp;		/* Temporary file descriptor */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction action;	/* Signal action */
+  struct sigaction oldaction;	/* Old signal action */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
@@ -101,12 +105,12 @@ main(int  argc,		/* I - Number of command-line arguments */
       switch (ch = argv[i][1])
       {
         case 'E' : /* Encrypt */
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
 	    cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
 #else
             fprintf(stderr, "%s: Sorry, no encryption support compiled in!\n",
 	            argv[0]);
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 	    break;
 
 	case '1' : /* TROFF font set 1 */
@@ -121,7 +125,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 
 	      if (i >= argc)
 	      {
-		fprintf(stderr, "lpr: Expected value after -%c option!\n", ch);
+		fprintf(stderr, "lpr: error - expected value after -%c option!\n", ch);
 		return (1);
 	      }
 	    }
@@ -133,7 +137,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	case 'n' : /* Ditroff */
 	case 't' : /* Troff */
 	case 'v' : /* Raster image */
-	    fprintf(stderr, "Warning: \'%c\' format modifier not supported - output may not be correct!\n",
+	    fprintf(stderr, "lpr: warning - \'%c\' format modifier not supported - output may not be correct!\n",
 	            ch);
 	    break;
 
@@ -145,7 +149,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	      i ++;
 	      if (i >= argc)
 	      {
-	        fputs("lpr: Expected option=value after -o option!\n", stderr);
+	        fputs("lpr: error - expected option=value after -o option!\n", stderr);
 		return (1);
 	      }
 
@@ -169,7 +173,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    break;
 
 	case 'm' : /* Mail on completion */
-	    fputs("Warning: email notification is not supported!\n", stderr);
+	    fputs("lpr: warning - email notification is not currently supported!\n", stderr);
 	    break;
 
 	case 'q' : /* Queue file but don't print */
@@ -189,7 +193,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	      i ++;
 	      if (i >= argc)
 	      {
-	        fputs("lpr: Expected destination after -P option!\n", stderr);
+	        fputs("lpr: error - expected destination after -P option!\n", stderr);
 		return (1);
 	      }
 
@@ -220,19 +224,13 @@ main(int  argc,		/* I - Number of command-line arguments */
 	      i ++;
 	      if (i >= argc)
 	      {
-	        fputs("lpr: Expected copy count after -# option!\n", stderr);
+	        fputs("lpr: error - expected copy count after -# option!\n", stderr);
 		return (1);
 	      }
 
 	      num_copies = atoi(argv[i]);
 	    }
 
-	    if (num_copies < 1 || num_copies > 100)
-	    {
-	      fputs("lpr: Number copies must be between 1 and 100.\n", stderr);
-	      return (1);
-	    }
-
             sprintf(buffer, "%d", num_copies);
             num_options = cupsAddOption("copies", buffer, num_options, &options);
 	    break;
@@ -247,7 +245,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	      i ++;
 	      if (i >= argc)
 	      {
-		fprintf(stderr, "lpr: Expected name after -%c option!\n", ch);
+		fprintf(stderr, "lpr: error - expected name after -%c option!\n", ch);
 		return (1);
 	      }
 
@@ -263,7 +261,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	      i ++;
 	      if (i >= argc)
 	      {
-	        fputs("lpr: Expected username after -U option!\n", stderr);
+	        fputs("lpr: error - expected username after -U option!\n", stderr);
 		return (1);
 	      }
 
@@ -272,7 +270,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    break;
 
 	default :
-	    fprintf(stderr, "lpr: Unknown option \'%c\'!\n", argv[i][1]);
+	    fprintf(stderr, "lpr: error - unknown option \'%c\'!\n", argv[i][1]);
 	    return (1);
       }
     else if (num_files < 1000)
@@ -281,6 +279,13 @@ main(int  argc,		/* I - Number of command-line arguments */
       * Print a file...
       */
 
+      if (access(argv[i], R_OK) != 0)
+      {
+        fprintf(stderr, "lpr: error - unable to access \"%s\" - %s\n", argv[i],
+	        strerror(errno));
+        return (1);
+      }
+
       files[num_files] = argv[i];
       num_files ++;
 
@@ -293,7 +298,7 @@ main(int  argc,		/* I - Number of command-line arguments */
       }
     }
     else
-      fprintf(stderr, "lpr: Too many files - \"%s\"\n", argv[i]);
+      fprintf(stderr, "lpr: error - too many files - \"%s\"\n", argv[i]);
  /*
   * See if we have any files to print; if not, print from stdin...
   */
@@ -303,23 +308,43 @@ main(int  argc,		/* I - Number of command-line arguments */
     if (num_dests == 0)
       num_dests = cupsGetDests(&dests);
 
-    for (j = 0, dest = dests; j < num_dests; j ++, dest ++)
-      if (dest->is_default)
-      {
-	printer = dests[j].name;
+    if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+    {
+      printer = dest->name;
 
-	for (j = 0; j < dest->num_options; j ++)
-	  if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
-	    num_options = cupsAddOption(dest->options[j].name,
-		                        dest->options[j].value,
-					num_options, &options);
-        break;
-      }
+      for (j = 0; j < dest->num_options; j ++)
+	if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
+	  num_options = cupsAddOption(dest->options[j].name,
+		                      dest->options[j].value,
+				      num_options, &options);
+    }
   }
 
   if (printer == NULL)
   {
-    fputs("lpr: error - no default destination available.\n", stderr);
+    val = NULL;
+
+    if ((printer = getenv("LPDEST")) == NULL)
+    {
+      if ((printer = getenv("PRINTER")) != NULL)
+      {
+        if (!strcmp(printer, "lp"))
+          printer = NULL;
+	else
+	  val = "PRINTER";
+      }
+    }
+    else
+      val = "LPDEST";
+
+    if (printer && !cupsGetDest(printer, NULL, num_dests, dests))
+      fprintf(stderr, "lpr: error - %s environment variable names non-existent destination \"%s\"!\n",
+              val, printer);
+    else if (cupsLastError() == IPP_NOT_FOUND)
+      fputs("lpr: error - no default destination available.\n", stderr);
+    else
+      fputs("lpr: error - scheduler not responding!\n", stderr);
+
     return (1);
   }
 
@@ -327,7 +352,7 @@ main(int  argc,		/* I - Number of command-line arguments */
   {
     job_id = cupsPrintFiles(printer, num_files, files, title, num_options, options);
 
-    if (deletefile)
+    if (deletefile && job_id > 0)
     {
      /*
       * Delete print files after printing...
@@ -344,25 +369,30 @@ main(int  argc,		/* I - Number of command-line arguments */
 #ifndef WIN32
 #  if defined(HAVE_SIGSET)
     sigset(SIGHUP, sighandler);
-    sigset(SIGINT, sighandler);
+    if (sigset(SIGINT, sighandler) == SIG_IGN)
+      sigset(SIGINT, SIG_IGN);
     sigset(SIGTERM, sighandler);
 #  elif defined(HAVE_SIGACTION)
     memset(&action, 0, sizeof(action));
     action.sa_handler = sighandler;
 
     sigaction(SIGHUP, &action, NULL);
-    sigaction(SIGINT, &action, NULL);
+    sigaction(SIGINT, NULL, &oldaction);
+    if (oldaction.sa_handler != SIG_IGN)
+      sigaction(SIGINT, &action, NULL);
     sigaction(SIGTERM, &action, NULL);
 #  else
     signal(SIGHUP, sighandler);
-    signal(SIGINT, sighandler);
+    if (signal(SIGINT, sighandler) == SIG_IGN)
+      signal(SIGINT, SIG_IGN);
     signal(SIGTERM, sighandler);
 #  endif
 #endif /* !WIN32 */
 
     if ((temp = cupsTempFd(tempfile, sizeof(tempfile))) < 0)
     {
-      fputs("lpr: unable to create temporary file.\n", stderr);
+      fprintf(stderr, "lpr: error - unable to create temporary file \"%s\" - %s\n",
+              tempfile, strerror(errno));
       return (1);
     }
 
@@ -374,7 +404,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 
     if (i == 0)
     {
-      fputs("lpr: stdin is empty, so no job has been sent.\n", stderr);
+      fputs("lpr: error - stdin is empty, so no job has been sent.\n", stderr);
       return (1);
     }
 
@@ -388,7 +418,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 
   if (job_id < 1)
   {
-    fprintf(stderr, "lpr: unable to print file: %s\n",
+    fprintf(stderr, "lpr: error - unable to print file: %s\n",
 	    ippErrorString(cupsLastError()));
     return (1);
   }
@@ -421,5 +451,5 @@ sighandler(int s)	/* I - Signal number */
 
 
 /*
- * End of "$Id: lpr.c,v 1.23 2002/01/02 17:58:35 mike Exp $".
+ * End of "$Id: lpr.c,v 1.36 2005/01/03 19:29:44 mike Exp $".
  */
