@@ -1,9 +1,9 @@
 /*
- * "$Id: language.h,v 1.24 2002/02/12 19:05:38 mike Exp $"
+ * "$Id: language.h,v 1.34 2005/01/03 19:29:45 mike Exp $"
  *
  *   Multi-language support for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,11 +15,13 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
+ *
+ *   This file is subject to the Apple OS-Developed Software exception.
  */
 
 #ifndef _CUPS_LANGUAGE_H_
@@ -150,6 +152,8 @@ typedef enum			/**** Message Indices ****/
   CUPS_MSG_TOP,
   CUPS_MSG_FILENAME,
   CUPS_MSG_PRINT,
+  CUPS_MSG_OPTIONS_INSTALLED,
+  CUPS_MSG_AUTO,
   CUPS_MSG_HTTP_BASE = 200,
   CUPS_MSG_HTTP_END = 505,
   CUPS_MSG_MAX
@@ -157,6 +161,7 @@ typedef enum			/**** Message Indices ****/
 
 typedef enum			/**** Language Encodings ****/
 {
+  CUPS_AUTO_ENCODING = -1,
   CUPS_US_ASCII,
   CUPS_ISO8859_1,
   CUPS_ISO8859_2,
@@ -201,18 +206,17 @@ typedef struct cups_lang_str	/**** Language Cache Structure ****/
  * Prototypes...
  */
 
-#  if defined(WIN32) || defined(__EMX__) || defined(__APPLE__)
-#    define		cupsLangDefault() cupsLangGet(setlocale(LC_ALL, ""))
-#  else
-#    define		cupsLangDefault() cupsLangGet(setlocale(LC_MESSAGES, ""))
-#  endif /* WIN32 || __EMX__ || __APPLE__ */
-
+#  define		cupsLangDefault() cupsLangGet(NULL)
 extern char		*cupsLangEncoding(cups_lang_t *lang);
 extern void		cupsLangFlush(void);
 extern void		cupsLangFree(cups_lang_t *lang);
 extern cups_lang_t	*cupsLangGet(const char *language);
 #  define		cupsLangString(lang,msg) (lang)->messages[(msg)]
 
+/**** New in CUPS 1.1.20 ****/
+extern void		_cupsRestoreLocale(int category, char *oldlocale);
+extern char		*_cupsSaveLocale(int category, const char *locale);
+
 #  ifdef __cplusplus
 }
 #  endif /* __cplusplus */
@@ -220,5 +224,5 @@ extern cups_lang_t	*cupsLangGet(const char *language);
 #endif /* !_CUPS_LANGUAGE_H_ */
 
 /*
- * End of "$Id: language.h,v 1.24 2002/02/12 19:05:38 mike Exp $".
+ * End of "$Id: language.h,v 1.34 2005/01/03 19:29:45 mike Exp $".
  */
