@@ -1,9 +1,9 @@
 /*
- * "$Id: common.h,v 1.9 2002/01/23 22:30:42 mike Exp $"
+ * "$Id: common.h,v 1.15 2005/01/03 19:29:57 mike Exp $"
  *
  *   Common filter definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,11 +15,13 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
+ *
+ *   This file is subject to the Apple OS-Developed Software exception.
  */
 
 /*
@@ -38,6 +40,15 @@
 #include <cups/string.h>
 
 
+/*
+ * C++ magic...
+ */
+
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
+
 /*
  * Globals...
  */
@@ -61,11 +72,21 @@ extern float	PageLeft,	/* Left margin */
 extern ppd_file_t *SetCommonOptions(int num_options, cups_option_t *options,
 		                    int change_size);
 extern void	UpdatePageVars(void);
+extern void	WriteCommon(void);
 extern void	WriteLabelProlog(const char *label, float bottom,
 		                 float top, float width);
 extern void	WriteLabels(int orient);
 
 
 /*
- * End of "$Id: common.h,v 1.9 2002/01/23 22:30:42 mike Exp $".
+ * C++ magic...
+ */
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+
+/*
+ * End of "$Id: common.h,v 1.15 2005/01/03 19:29:57 mike Exp $".
  */
