@@ -1,9 +1,9 @@
 /*
- * "$Id: hpgl-vector.c,v 1.15 2002/01/02 17:58:58 mike Exp $"
+ * "$Id: hpgl-vector.c,v 1.22 2005/01/03 19:29:57 mike Exp $"
  *
  *   HP-GL/2 vector routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,12 +15,14 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
  *   AA_arc_absolute()    - Draw an arc.
@@ -98,6 +100,8 @@ AA_arc_absolute(int     num_params,	/* I - Number of parameters */
     if (!PolygonMode)
       Outputf("MP\n");
 
+    PenValid = 1;
+
     Outputf("%.3f %.3f MO\n", PenPosition[0], PenPosition[1]);
 
     if (start < end)
@@ -178,6 +182,8 @@ AR_arc_relative(int     num_params,	/* I - Number of parameters */
     if (!PolygonMode)
       Outputf("MP\n");
 
+    PenValid = 1;
+
     Outputf("%.3f %.3f MO\n", PenPosition[0], PenPosition[1]);
 
     if (start < end)
@@ -232,6 +238,8 @@ AT_arc_absolute3(int     num_params,	/* I - Number of parameters */
     if (!PolygonMode)
       Outputf("MP\n");
 
+    PenValid = 1;
+
     Outputf("%.3f %.3f MO\n", PenPosition[0], PenPosition[1]);
 
     PenPosition[0] = Transform[0][0] * params[0].value.number +
@@ -291,6 +299,8 @@ CI_circle(int     num_params,	/* I - Number of parameters */
   if (!PolygonMode)
     Outputf("MP\n");
 
+  PenValid = 1;
+
   for (theta = 0.0; theta < 360.0; theta += dt)
   {
     x = (float)(PenPosition[0] +
@@ -359,7 +369,7 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
 
   base_bits = 6;
   frac_bits = 1.0;
-  draw      = 1;
+  draw      = PenDown;
   abscoords = 0;
 
   if (num_params == 0)
@@ -368,7 +378,13 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
   if (!PolygonMode)
   {
     Outputf("MP\n");
+    PenValid = 0;
+  }
+
+  if (!PenValid)
+  {
     Outputf("%.3f %.3f MO\n", PenPosition[0], PenPosition[1]);
+    PenValid = 1;
   }
 
   for (s = (unsigned char *)params[0].value.string; *s != '\0';)
@@ -381,18 +397,23 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
 #ifdef DEBUG
           fputs("DEBUG:     7-bit\n", stderr);
 #endif /* DEBUG */
+
+          Outputf("%% PE: 7-bit\n");
           break;
       case ':' :	/* Select pen */
           s ++;
           PenNumber = (int)decode_number(&s, base_bits, 1.0);
-	  if (PageDirty)
-	    printf("%.3f %.3f %.3f %.2f SP\n", Pens[PenNumber].rgb[0],
-		   Pens[PenNumber].rgb[1], Pens[PenNumber].rgb[2],
-		   Pens[PenNumber].width * PenScaling);
 
 #ifdef DEBUG
           fprintf(stderr, "DEBUG:     set pen #%d\n", PenNumber);
 #endif /* DEBUG */
+
+          Outputf("%% PE: set pen #%d\n", PenNumber);
+
+	  if (PageDirty)
+	    printf("%.3f %.3f %.3f %.2f SP\n", Pens[PenNumber].rgb[0],
+		   Pens[PenNumber].rgb[1], Pens[PenNumber].rgb[2],
+		   Pens[PenNumber].width * PenScaling);
           break;
       case '<' :	/* Next coords are a move-to */
           draw = 0;
@@ -401,6 +422,8 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
 #ifdef DEBUG
           fputs("DEBUG:     moveto\n", stderr);
 #endif /* DEBUG */
+
+	  Outputf("%% PE: moveto\n");
           break;
       case '>' :	/* Set fractional bits */
           s ++;
@@ -410,6 +433,8 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
 #ifdef DEBUG
           fprintf(stderr, "DEBUG:     set fractional bits %d\n", temp);
 #endif /* DEBUG */
+
+          Outputf("%% PE: set fractional bits %d\n", temp);
           break;
       case '=' :	/* Next coords are absolute */
           s ++;
@@ -418,6 +443,8 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
 #ifdef DEBUG
           fputs("DEBUG:     absolute\n", stderr);
 #endif /* DEBUG */
+
+          Outputf("%% PE: absolute\n");
           break;
       default :
           if (*s >= 63)
@@ -433,6 +460,8 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
             fprintf(stderr, "DEBUG:     coords %.3f %.3f\n", x, y);
 #endif /* DEBUG */
 
+            Outputf("%% PE: coords %.3f %.3f\n", x, y);
+
             if (abscoords)
             {
 	      tx = Transform[0][0] * x + Transform[0][1] * y +
@@ -454,8 +483,12 @@ PE_polyline_encoded(int     num_params,	/* I - Number of parameters */
 	    }
 
             if (draw)
-              Outputf("%.3f %.3f LI\n", tx, ty);
-            else
+	    {
+	      if (fabs(PenPosition[0] - tx) > 0.001 ||
+        	  fabs(PenPosition[1] - ty) > 0.001)
+        	Outputf("%.3f %.3f LI\n", tx, ty);
+            }
+	    else
               Outputf("%.3f %.3f MO\n", tx, ty);
 
 	    PenPosition[0] = (float)tx;
@@ -534,6 +567,8 @@ RT_arc_relative3(int     num_params,	/* I - Number of parameters */
     if (!PolygonMode)
       Outputf("MP\n");
 
+    PenValid = 1;
+
     Outputf("%.3f %.3f MO\n", PenPosition[0], PenPosition[1]);
 
     PenPosition[0] = Transform[0][0] * params[0].value.number +
@@ -600,7 +635,7 @@ decode_number(unsigned char **s,	/* IO - String to decode */
       else if (**s < 63)
       {
         if (**s != '\r' && **s != '\n')
-          fprintf(stderr, "hpgl2ps: Bad PE character \'%c\'!\n", **s);
+          fprintf(stderr, "ERROR: Bad PE character 0x%02X!\n", **s);
 
         continue;
       }
@@ -642,7 +677,7 @@ decode_number(unsigned char **s,	/* IO - String to decode */
       else if (**s < 63)
       {
         if (**s != '\r' && **s != '\n')
-          fprintf(stderr, "hpgl2ps: Bad PE character \'%c\'!\n", **s);
+          fprintf(stderr, "ERROR: Bad PE character 0x%02X!\n", **s);
 
         continue;
       }
@@ -685,9 +720,12 @@ plot_points(int     num_params,	/* I - Number of parameters */
   if (PenDown)
   {
     if (!PolygonMode)
+    {
       Outputf("MP\n");
+      Outputf("%.3f %.3f MO\n", PenPosition[0], PenPosition[1]);
 
-    Outputf("%.3f %.3f MO\n", PenPosition[0], PenPosition[1]);
+      PenValid = 1;
+    }
   }
 
   for (i = 0; i < num_params; i += 2)
@@ -712,7 +750,13 @@ plot_points(int     num_params,	/* I - Number of parameters */
     }
 
     if (PenDown)
-      Outputf("%.3f %.3f LI\n", x, y);
+    {
+      if (PolygonMode && i == 0)
+        Outputf("%.3f %.3f MO\n", x, y);
+      else if (fabs(PenPosition[0] - x) > 0.001 ||
+               fabs(PenPosition[1] - y) > 0.001)
+        Outputf("%.3f %.3f LI\n", x, y);
+    }
 
     PenPosition[0] = x;
     PenPosition[1] = y;
@@ -727,5 +771,5 @@ plot_points(int     num_params,	/* I - Number of parameters */
 
 
 /*
- * End of "$Id: hpgl-vector.c,v 1.15 2002/01/02 17:58:58 mike Exp $".
+ * End of "$Id: hpgl-vector.c,v 1.22 2005/01/03 19:29:57 mike Exp $".
  */
