@@ -1,9 +1,9 @@
 /*
- * "$Id: image.h,v 1.12 2002/01/02 17:59:00 mike Exp $"
+ * "$Id: image.h,v 1.19 2005/01/03 19:29:57 mike Exp $"
  *
  *   Image library definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,11 +15,13 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
+ *
+ *   This file is subject to the Apple OS-Developed Software exception.
  */
 
 #ifndef _IMAGE_H_
@@ -34,8 +36,16 @@
 #  include <string.h>
 #  include <errno.h>
 #  include <config.h>
+#  include "raster.h"
 
 
+/*
+ * Maximum image dimensions that we can handle...
+ */
+
+#  define IMAGE_MAX_WIDTH	0x07ffffff	/* 2^27-1 to allow for 15-channel data */
+#  define IMAGE_MAX_HEIGHT	0x7fffffff	/* 2^31-1 */
+
 /*
  * Colorspaces...
  */
@@ -45,6 +55,7 @@
 #  define IMAGE_BLACK	-1		/* Black */
 #  define IMAGE_WHITE	1		/* White (luminance) */
 #  define IMAGE_RGB	3		/* Red, green, and blue */
+#  define IMAGE_RGB_CMYK 4		/* Use RGB or CMYK */
 
 /*
  * Tile definitions...
@@ -155,6 +166,7 @@ typedef struct
 extern image_t	*ImageOpen(char *filename, int primary, int secondary,
 		           int saturation, int hue, const ib_t *lut);
 extern void	ImageClose(image_t *img);
+extern void	ImageSetColorSpace(cups_cspace_t cs);
 extern void	ImageSetMaxTiles(image_t *img, int max_tiles);
 extern void	ImageSetProfile(float d, float g, float matrix[3][3]);
 
@@ -209,6 +221,12 @@ extern void	ImageRGBToBlack(const ib_t *in, ib_t *out, int count);
 extern void	ImageRGBToCMY(const ib_t *in, ib_t *out, int count);
 extern void	ImageRGBToCMYK(const ib_t *in, ib_t *out, int count);
 
+extern void	ImageCMYKToWhite(const ib_t *in, ib_t *out, int count);
+extern void	ImageCMYKToRGB(const ib_t *in, ib_t *out, int count);
+extern void	ImageCMYKToBlack(const ib_t *in, ib_t *out, int count);
+extern void	ImageCMYKToCMY(const ib_t *in, ib_t *out, int count);
+extern void	ImageCMYKToCMYK(const ib_t *in, ib_t *out, int count);
+
 extern void	ImageRGBAdjust(ib_t *pixels, int count, int saturation, int hue);
 
 extern void	ImageLut(ib_t *pixels, int count, const ib_t *lut);
@@ -227,5 +245,5 @@ extern void	ImageZoomFree(izoom_t *z);
 #endif /* !_IMAGE_H_ */
 
 /*
- * End of "$Id: image.h,v 1.12 2002/01/02 17:59:00 mike Exp $".
+ * End of "$Id: image.h,v 1.19 2005/01/03 19:29:57 mike Exp $".
  */
