@@ -1,9 +1,9 @@
 /*
- * "$Id: textcommon.c,v 1.20 2002/01/17 21:39:26 mike Exp $"
+ * "$Id: textcommon.c,v 1.33 2005/01/03 19:29:58 mike Exp $"
  *
  *   Common text filter routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,12 +15,14 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
  *   TextMain()         - Standard main entry for text filters.
@@ -76,11 +78,13 @@ static char *code_keywords[] =	/* List of known C/C++ keywords... */
 	  "class",
 	  "compl",
 	  "const",
+	  "const_cast",
 	  "continue",
 	  "default",
 	  "delete",
 	  "do",
 	  "double",
+	  "dynamic_cast",
 	  "else",
 	  "enum",
 	  "explicit",
@@ -106,11 +110,13 @@ static char *code_keywords[] =	/* List of known C/C++ keywords... */
 	  "protected",
 	  "public",
 	  "register",
+	  "reinterpret_cast",
 	  "return",
 	  "short",
 	  "signed",
 	  "sizeof",
 	  "static",
+	  "static_cast",
 	  "struct",
 	  "switch",
 	  "template",
@@ -549,7 +555,9 @@ TextMain(const char *name,	/* I - Name of filter */
   options     = NULL;
   num_options = cupsParseOptions(argv[5], 0, &options);
 
-  if ((val = cupsGetOption("prettyprint", num_options, options)) != NULL)
+  if ((val = cupsGetOption("prettyprint", num_options, options)) != NULL &&
+      strcasecmp(val, "no") && strcasecmp(val, "off") &&
+      strcasecmp(val, "false"))
   {
     PageLeft     = 72.0f;
     PageRight    = PageWidth - 36.0f;
@@ -592,7 +600,11 @@ TextMain(const char *name,	/* I - Name of filter */
 
   ppd = SetCommonOptions(num_options, options, 1);
 
-  WrapLines = cupsGetOption("nowrap", num_options, options) == NULL;
+  if ((val = cupsGetOption("wrap", num_options, options)) == NULL)
+    WrapLines = 1;
+  else
+    WrapLines = !strcasecmp(val, "true") || !strcasecmp(val, "on") ||
+                !strcasecmp(val, "yes");
 
   if ((val = cupsGetOption("columns", num_options, options)) != NULL)
     PageColumns = atoi(val);
@@ -603,7 +615,7 @@ TextMain(const char *name,	/* I - Name of filter */
   if ((val = cupsGetOption("lpi", num_options, options)) != NULL)
     LinesPerInch = atof(val);
 
-  if ((val = cupsGetOption("prettyprint", num_options, options)) != NULL)
+  if (PrettyPrint)
     PageTop -= 216.0f / LinesPerInch;
 
   Copies = atoi(argv[4]);
@@ -697,6 +709,30 @@ TextMain(const char *name,	/* I - Name of filter */
 	  keycol = column;
           break;
 
+      case 0x0d :		/* CR */
+#ifndef __APPLE__
+         /*
+	  * All but MacOS/Darwin treat CR as was intended by ANSI
+	  * folks, namely to move to column 0/1.  Some programs still
+	  * use this to do boldfacing and underlining...
+	  */
+
+          column = 0;
+          break;
+#else
+         /*
+	  * MacOS/Darwin still need to treat CR as a line ending.
+	  */
+
+          {
+	    int nextch;
+            if ((nextch = getc(fp)) != 0x0a)
+	      ungetc(nextch, fp);
+	    else
+	      ch = nextch;
+	  }
+#endif /* !__APPLE__ */
+
       case 0x0a :		/* LF - output current line */
           if (PrettyPrint && keyptr > keyword)
 	  {
@@ -789,10 +825,6 @@ TextMain(const char *name,	/* I - Name of filter */
           }
           break;
 
-      case 0x0d :		/* CR */
-          column = 0;
-          break;
-
       case 0x1b :		/* Escape sequence */
           ch = getutf8(fp);
 	  if (ch == '7')
@@ -857,7 +889,7 @@ TextMain(const char *name,	/* I - Name of filter */
 
 	      attr &= ~ATTR_BOLD;
 	    }
-	    else if (!(isalnum(ch) || ch == '_') && keyptr > keyword)
+	    else if (!(isalnum(ch & 255) || ch == '_') && keyptr > keyword)
 	    {
 	     /*
 	      * Look for a keyword...
@@ -883,7 +915,7 @@ TextMain(const char *name,	/* I - Name of filter */
 		}
 	      }
 	    }
-	    else if ((isalnum(ch) || ch == '_') && !ccomment && !cstring)
+	    else if ((isalnum(ch & 255) || ch == '_') && !ccomment && !cstring)
 	    {
 	     /*
 	      * Add characters to the current keyword (if they'll fit).
@@ -1151,5 +1183,5 @@ getutf8(FILE *fp)	/* I - File to read from */
 
 
 /*
- * End of "$Id: textcommon.c,v 1.20 2002/01/17 21:39:26 mike Exp $".
+ * End of "$Id: textcommon.c,v 1.33 2005/01/03 19:29:58 mike Exp $".
  */
