@@ -1,9 +1,9 @@
 /*
- * "$Id: auth.c,v 1.55 2002/01/02 17:59:13 mike Exp $"
+ * "$Id: auth.c,v 1.77 2005/01/03 19:29:59 mike Exp $"
  *
  *   Authorization routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -39,12 +39,12 @@
  *   FindBest()           - Find the location entry that best matches the
  *                          resource.
  *   FindLocation()       - Find the named location.
+ *   GetMD5Passwd()       - Get an MD5 password.
  *   IsAuthorized()       - Check to see if the user is authorized...
  *   add_allow()          - Add an allow mask to the location.
  *   add_deny()           - Add a deny mask to the location.
  *   cups_crypt()         - Encrypt the password using the DES or MD5
  *                          algorithms, as needed.
- *   get_md5_passwd()     - Get an MD5 password.
  *   pam_func()           - PAM conversation function.
  *   to64()               - Base64-encode an integer value...
  */
@@ -64,7 +64,11 @@
 #  include <crypt.h>
 #endif /* HAVE_CRYPT_H */
 #if HAVE_LIBPAM
-#  include <security/pam_appl.h>
+#  ifdef HAVE_PAM_PAM_APPL_H
+#    include <pam/pam_appl.h>
+#  else
+#    include <security/pam_appl.h>
+#  endif /* HAVE_PAM_PAM_APPL_H */
 #endif /* HAVE_LIBPAM */
 #ifdef HAVE_USERSEC_H
 #  include <usersec.h>
@@ -80,8 +84,6 @@ static authmask_t	*add_deny(location_t *loc);
 #if !HAVE_LIBPAM
 static char		*cups_crypt(const char *pw, const char *salt);
 #endif /* !HAVE_LIBPAM */
-static char		*get_md5_passwd(const char *username, const char *group,
-			                char passwd[33]);
 #if HAVE_LIBPAM
 static int		pam_func(int, const struct pam_message **,
 			         struct pam_response **, void *);
@@ -94,9 +96,9 @@ static void		to64(char *s, unsigned long v, int n);
  * Local globals...
  */
 
-#ifdef __hpux
+#if defined(__hpux) && defined(HAVE_LIBPAM)
 static client_t		*auth_client;	/* Current client being authenticated */
-#endif /* __hpux */
+#endif /* __hpux && HAVE_LIBPAM */
 
 
 /*
@@ -130,7 +132,7 @@ AddLocation(const char *location)	/* I - Location path */
   */
 
   memset(temp, 0, sizeof(location_t));
-  strncpy(temp->location, location, sizeof(temp->location) - 1);
+  strlcpy(temp->location, location, sizeof(temp->location));
   temp->length = strlen(temp->location);
 
   LogMessage(L_DEBUG, "AddLocation: added location \'%s\'", location);
@@ -188,14 +190,53 @@ AllowHost(location_t *loc,	/* I - Location to add to */
           char       *name)	/* I - Name of host or domain to add */
 {
   authmask_t	*temp;		/* New host/domain mask */
+  char		ifname[32],	/* Interface name */
+		*ifptr;		/* Pointer to end of name */
 
 
   if ((temp = add_allow(loc)) == NULL)
     return;
 
-  temp->type             = AUTH_NAME;
-  temp->mask.name.name   = strdup(name);
-  temp->mask.name.length = strlen(name);
+  if (strcasecmp(name, "@LOCAL") == 0)
+  {
+   /*
+    * Allow *interface*...
+    */
+
+    temp->type             = AUTH_INTERFACE;
+    temp->mask.name.name   = strdup("*");
+    temp->mask.name.length = 1;
+  }
+  else if (strncasecmp(name, "@IF(", 4) == 0)
+  {
+   /*
+    * Allow *interface*...
+    */
+
+    strlcpy(ifname, name + 4, sizeof(ifname));
+
+    ifptr = ifname + strlen(ifname);
+
+    if (ifptr[-1] == ')')
+    {
+      ifptr --;
+      *ifptr = '\0';
+    }
+
+    temp->type             = AUTH_INTERFACE;
+    temp->mask.name.name   = strdup(ifname);
+    temp->mask.name.length = ifptr - ifname;
+  }
+  else
+  {
+   /*
+    * Allow name...
+    */
+
+    temp->type             = AUTH_NAME;
+    temp->mask.name.name   = strdup(name);
+    temp->mask.name.length = strlen(name);
+  }
 
   LogMessage(L_DEBUG, "AllowHost: %s allow %s", loc->location, name);
 }
@@ -237,10 +278,60 @@ CheckAuth(unsigned   ip,	/* I - Client address */
           int        num_masks, /* I - Number of masks */
           authmask_t *masks)	/* I - Masks */
 {
+  cups_netif_t	*iface;		/* Network interface */
+  unsigned	netip;		/* Network address */
+
+
   while (num_masks > 0)
   {
     switch (masks->type)
     {
+      case AUTH_INTERFACE :
+         /*
+	  * Check for a match with a network interface...
+	  */
+
+          netip = htonl(ip);
+
+          if (strcmp(masks->mask.name.name, "*") == 0)
+	  {
+	   /*
+	    * Check against all local interfaces...
+	    */
+
+            NetIFUpdate();
+
+	    for (iface = NetIFList; iface != NULL; iface = iface->next)
+	    {
+	     /*
+	      * Only check local interfaces...
+	      */
+
+	      if (!iface->is_local)
+	        continue;
+
+              if ((netip & iface->mask.sin_addr.s_addr) ==
+	          (iface->address.sin_addr.s_addr &
+		   iface->mask.sin_addr.s_addr))
+		return (1);
+	    }
+	  }
+	  else
+	  {
+	   /*
+	    * Check the named interface...
+	    */
+
+            if ((iface = NetIFFind(masks->mask.name.name)) != NULL)
+	    {
+              if ((netip & iface->mask.sin_addr.s_addr) ==
+	          (iface->address.sin_addr.s_addr &
+		   iface->mask.sin_addr.s_addr))
+		return (1);
+            }
+	  }
+	  break;
+
       case AUTH_NAME :
          /*
 	  * Check for exact name match...
@@ -288,6 +379,7 @@ CopyLocation(location_t **loc)	/* IO - Original location */
   int		i;		/* Looping var */
   int		locindex;	/* Index into Locations array */
   location_t	*temp;		/* New location */
+  char		location[HTTP_MAX_URI];	/* Location of resource */
 
 
  /*
@@ -297,7 +389,14 @@ CopyLocation(location_t **loc)	/* IO - Original location */
 
   locindex = *loc - Locations;
 
-  if ((temp = AddLocation((*loc)->location)) == NULL)
+ /*
+  * Use a local copy of location because AddLocation may cause
+  * this memory to be moved...
+  */
+
+  strlcpy(location, (*loc)->location, sizeof(location));
+
+  if ((temp = AddLocation(location)) == NULL)
     return (NULL);
 
   *loc = Locations + locindex;
@@ -439,14 +538,14 @@ DeleteAllLocations(void)
       free(loc->names);
 
     for (j = loc->num_allow, mask = loc->allow; j > 0; j --, mask ++)
-      if (mask->type == AUTH_NAME)
+      if (mask->type == AUTH_NAME || mask->type == AUTH_INTERFACE)
         free(mask->mask.name.name);
 
     if (loc->num_allow > 0)
       free(loc->allow);
 
     for (j = loc->num_deny, mask = loc->deny; j > 0; j --, mask ++)
-      if (mask->type == AUTH_NAME)
+      if (mask->type == AUTH_NAME || mask->type == AUTH_INTERFACE)
         free(mask->mask.name.name);
 
     if (loc->num_deny > 0)
@@ -474,14 +573,53 @@ DenyHost(location_t *loc,	/* I - Location to add to */
          char       *name)	/* I - Name of host or domain to add */
 {
   authmask_t	*temp;		/* New host/domain mask */
+  char		ifname[32],	/* Interface name */
+		*ifptr;		/* Pointer to end of name */
 
 
   if ((temp = add_deny(loc)) == NULL)
     return;
 
-  temp->type             = AUTH_NAME;
-  temp->mask.name.name   = strdup(name);
-  temp->mask.name.length = strlen(name);
+  if (strcasecmp(name, "@LOCAL") == 0)
+  {
+   /*
+    * Deny *interface*...
+    */
+
+    temp->type             = AUTH_INTERFACE;
+    temp->mask.name.name   = strdup("*");
+    temp->mask.name.length = 1;
+  }
+  else if (strncasecmp(name, "@IF(", 4) == 0)
+  {
+   /*
+    * Deny *interface*...
+    */
+
+    strlcpy(ifname, name + 4, sizeof(ifname));
+
+    ifptr = ifname + strlen(ifname);
+
+    if (ifptr[-1] == ')')
+    {
+      ifptr --;
+      *ifptr = '\0';
+    }
+
+    temp->type             = AUTH_INTERFACE;
+    temp->mask.name.name   = strdup(ifname);
+    temp->mask.name.length = ifptr - ifname;
+  }
+  else
+  {
+   /*
+    * Deny name...
+    */
+
+    temp->type             = AUTH_NAME;
+    temp->mask.name.name   = strdup(name);
+    temp->mask.name.length = strlen(name);
+  }
 
   LogMessage(L_DEBUG, "DenyHost: %s deny %s", loc->location, name);
 }
@@ -528,7 +666,7 @@ FindBest(const char   *path,	/* I - Resource path */
 		*best;		/* Best match for location so far */
   int		bestlen;	/* Length of best match */
   int		limit;		/* Limit field */
-  static int	limits[] =	/* Map http_status_t to AUTH_LIMIT_xyz */
+  static const int limits[] =	/* Map http_status_t to AUTH_LIMIT_xyz */
 		{
 		  AUTH_LIMIT_ALL,
 		  AUTH_LIMIT_OPTIONS,
@@ -553,8 +691,7 @@ FindBest(const char   *path,	/* I - Resource path */
   * URIs...
   */
 
-  strncpy(uri, path, sizeof(uri) - 1);
-  uri[sizeof(uri) - 1] = '\0';
+  strlcpy(uri, path, sizeof(uri));
 
   if (strncmp(uri, "/printers/", 10) == 0 ||
       strncmp(uri, "/classes/", 9) == 0)
@@ -584,13 +721,35 @@ FindBest(const char   *path,	/* I - Resource path */
     LogMessage(L_DEBUG2, "FindBest: Location %s Limit %x",
                loc->location, loc->limit);
 
-    if (loc->length > bestlen &&
-        strncmp(uri, loc->location, loc->length) == 0 &&
-	loc->location[0] == '/' &&
-	(limit & loc->limit) != 0)
+    if (!strncmp(uri, "/printers/", 10) ||!strncmp(uri, "/classes/", 9))
     {
-      best    = loc;
-      bestlen = loc->length;
+     /*
+      * Use case-insensitive comparison for queue names...
+      */
+
+      if (loc->length > bestlen &&
+          strncasecmp(uri, loc->location, loc->length) == 0 &&
+	  loc->location[0] == '/' &&
+	  (limit & loc->limit) != 0)
+      {
+	best    = loc;
+	bestlen = loc->length;
+      }
+    }
+    else
+    {
+     /*
+      * Use case-sensitive comparison for other URIs...
+      */
+
+      if (loc->length > bestlen &&
+          strncmp(uri, loc->location, loc->length) == 0 &&
+	  loc->location[0] == '/' &&
+	  (limit & loc->limit) != 0)
+      {
+	best    = loc;
+	bestlen = loc->length;
+      }
     }
   }
 
@@ -627,6 +786,64 @@ FindLocation(const char *location)	/* I - Connection */
 }
 
 
+/*
+ * 'GetMD5Passwd()' - Get an MD5 password.
+ */
+
+char *					/* O - MD5 password string */
+GetMD5Passwd(const char *username,	/* I - Username */
+             const char *group,		/* I - Group */
+             char       passwd[33])	/* O - MD5 password string */
+{
+  cups_file_t	*fp;			/* passwd.md5 file */
+  char		filename[1024],		/* passwd.md5 filename */
+		line[256],		/* Line from file */
+		tempuser[33],		/* User from file */
+		tempgroup[33];		/* Group from file */
+
+
+  LogMessage(L_DEBUG2, "GetMD5Passwd(username=\"%s\", group=\"%s\", passwd=%p)",
+             username, group ? group : "(null)", passwd);
+
+  snprintf(filename, sizeof(filename), "%s/passwd.md5", ServerRoot);
+  if ((fp = cupsFileOpen(filename, "r")) == NULL)
+  {
+    LogMessage(L_ERROR, "Unable to open %s - %s", filename, strerror(errno));
+    return (NULL);
+  }
+
+  while (cupsFileGets(fp, line, sizeof(line)) != NULL)
+  {
+    if (sscanf(line, "%32[^:]:%32[^:]:%32s", tempuser, tempgroup, passwd) != 3)
+    {
+      LogMessage(L_ERROR, "Bad MD5 password line: %s", line);
+      continue;
+    }
+
+    if (strcmp(username, tempuser) == 0 &&
+        (group == NULL || strcmp(group, tempgroup) == 0))
+    {
+     /*
+      * Found the password entry!
+      */
+
+      LogMessage(L_DEBUG2, "Found MD5 user %s, group %s...", username,
+                 tempgroup);
+
+      cupsFileClose(fp);
+      return (passwd);
+    }
+  }
+
+ /*
+  * Didn't find a password entry - return NULL!
+  */
+
+  cupsFileClose(fp);
+  return (NULL);
+}
+
+
 /*
  * 'IsAuthorized()' - Check to see if the user is authorized...
  */
@@ -659,7 +876,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
   struct spwd	*spw;		/* Shadow password data */
 #  endif /* HAVE_SHADOW_H */
 #endif /* HAVE_LIBPAM */
-  static const char *states[] =	/* HTTP client states... */
+  static const char * const states[] =	/* HTTP client states... */
 		{
 		  "WAITING",
 		  "OPTIONS",
@@ -755,7 +972,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
   if (auth == AUTH_DENY && best->satisfy == AUTH_SATISFY_ALL)
     return (HTTP_FORBIDDEN);
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
  /*
   * See if encryption is required...
   */
@@ -765,7 +982,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
     LogMessage(L_DEBUG2, "IsAuthorized: Need upgrade to TLS...");
     return (HTTP_UPGRADE_REQUIRED);
   }
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 
  /*
   * Now see what access level is required...
@@ -775,7 +992,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
     return (HTTP_OK);
 
   LogMessage(L_DEBUG2, "IsAuthorized: username = \"%s\" password = %d chars",
-	     con->username, strlen(con->password));
+	     con->username, (int)strlen(con->password));
   DEBUG_printf(("IsAuthorized: username = \"%s\", password = \"%s\"\n",
 		con->username, con->password));
 
@@ -818,16 +1035,9 @@ IsAuthorized(client_t *con)	/* I - Connection */
 	  * Get the user info...
 	  */
 
-	  pw = getpwnam(con->username);	/* Get the current password */
+	  pw = getpwnam(con->username);		/* Get the current password */
 	  endpwent();				/* Close the password file */
 
-	  if (pw == NULL)			/* No such user... */
-	  {
-	    LogMessage(L_WARN, "IsAuthorized: Unknown username \"%s\"; access denied.",
-        	       con->username);
-	    return (HTTP_UNAUTHORIZED);
-	  }
-
 #if HAVE_LIBPAM
 	 /*
 	  * Only use PAM to do authentication.  This allows MD5 passwords, among
@@ -892,6 +1102,17 @@ IsAuthorized(client_t *con)	/* I - Connection */
 	    return (HTTP_UNAUTHORIZED);
 	  }
 #else
+         /*
+	  * Use normal UNIX password file-based authentication...
+	  */
+
+	  if (pw == NULL)			/* No such user... */
+	  {
+	    LogMessage(L_WARN, "IsAuthorized: Unknown username \"%s\"; access denied.",
+        	       con->username);
+	    return (HTTP_UNAUTHORIZED);
+	  }
+
 #  ifdef HAVE_SHADOW_H
 	  spw = getspnam(con->username);
 	  endspent();
@@ -974,20 +1195,22 @@ IsAuthorized(client_t *con)	/* I - Connection */
 
 	  if (best->num_names && best->level == AUTH_GROUP)
 	  {
+	    LogMessage(L_DEBUG2, "IsAuthorized: num_names = %d", best->num_names);
+
             for (i = 0; i < best->num_names; i ++)
-	      if (get_md5_passwd(con->username, best->names[i], md5))
+	      if (GetMD5Passwd(con->username, best->names[i], md5))
 		break;
 
             if (i >= best->num_names)
 	      md5[0] = '\0';
 	  }
-	  else if (!get_md5_passwd(con->username, NULL, md5))
+	  else if (!GetMD5Passwd(con->username, NULL, md5))
 	    md5[0] = '\0';
 
 
 	  if (!md5[0])
 	  {
-            LogMessage(L_ERROR, "IsAuthorized: No matching user:group for \"%s\" in passwd.md5!",
+            LogMessage(L_DEBUG2, "IsAuthorized: No matching user:group for \"%s\" in passwd.md5!",
 	               con->username);
             return (HTTP_UNAUTHORIZED);
 	  }
@@ -996,7 +1219,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
 
 	  if (strcmp(md5, con->password) != 0)
 	  {
-            LogMessage(L_ERROR, "IsAuthorized: MD5s \"%s\" and \"%s\" don't match!",
+            LogMessage(L_DEBUG2, "IsAuthorized: MD5s \"%s\" and \"%s\" don't match!",
 	               md5, con->password);
             return (HTTP_UNAUTHORIZED);
 	  }
@@ -1009,19 +1232,21 @@ IsAuthorized(client_t *con)	/* I - Connection */
 
 	  if (best->num_names && best->level == AUTH_GROUP)
 	  {
+	    LogMessage(L_DEBUG2, "IsAuthorized: num_names = %d", best->num_names);
+
             for (i = 0; i < best->num_names; i ++)
-	      if (get_md5_passwd(con->username, best->names[i], md5))
+	      if (GetMD5Passwd(con->username, best->names[i], md5))
 		break;
 
             if (i >= best->num_names)
 	      md5[0] = '\0';
 	  }
-	  else if (!get_md5_passwd(con->username, NULL, md5))
+	  else if (!GetMD5Passwd(con->username, NULL, md5))
 	    md5[0] = '\0';
 
 	  if (!md5[0])
 	  {
-            LogMessage(L_ERROR, "IsAuthorized: No matching user:group for \"%s\" in passwd.md5!",
+            LogMessage(L_DEBUG2, "IsAuthorized: No matching user:group for \"%s\" in passwd.md5!",
 	               con->username);
             return (HTTP_UNAUTHORIZED);
 	  }
@@ -1030,7 +1255,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
 
 	  if (strcmp(md5, basicmd5) != 0)
 	  {
-            LogMessage(L_ERROR, "IsAuthorized: MD5s \"%s\" and \"%s\" don't match!",
+            LogMessage(L_DEBUG2, "IsAuthorized: MD5s \"%s\" and \"%s\" don't match!",
 	               md5, basicmd5);
             return (HTTP_UNAUTHORIZED);
 	  }
@@ -1107,7 +1332,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
       * Check to see if the default group ID matches for the user...
       */
 
-      if (grp->gr_gid == pw->pw_gid)
+      if (pw != NULL && grp->gr_gid == pw->pw_gid)
 	return (HTTP_OK);
     }
 
@@ -1257,20 +1482,20 @@ cups_crypt(const char *pw,	/* I - Password string */
     pwlen = strlen(pw);
 
     md5_init(&state);
-    md5_append(&state, pw, pwlen);
-    md5_append(&state, salt, salt_end - salt);
+    md5_append(&state, (md5_byte_t *)pw, pwlen);
+    md5_append(&state, (md5_byte_t *)salt, salt_end - salt);
 
     md5_init(&state2);
-    md5_append(&state2, pw, pwlen);
-    md5_append(&state2, salt + 3, salt_end - salt - 3);
-    md5_append(&state2, pw, pwlen);
+    md5_append(&state2, (md5_byte_t *)pw, pwlen);
+    md5_append(&state2, (md5_byte_t *)salt + 3, salt_end - salt - 3);
+    md5_append(&state2, (md5_byte_t *)pw, pwlen);
     md5_finish(&state2, digest);
 
     for (i = pwlen; i > 0; i -= 16)
       md5_append(&state, digest, i > 16 ? 16 : i);
 
     for (i = pwlen; i > 0; i >>= 1)
-      md5_append(&state, (i & 1) ? "" : pw, 1);
+      md5_append(&state, (md5_byte_t *)((i & 1) ? "" : pw), 1);
 
     md5_finish(&state, digest);
 
@@ -1279,20 +1504,20 @@ cups_crypt(const char *pw,	/* I - Password string */
       md5_init(&state);
 
       if (i & 1)
-        md5_append(&state, pw, pwlen);
+        md5_append(&state, (md5_byte_t *)pw, pwlen);
       else
         md5_append(&state, digest, 16);
 
       if (i % 3)
-        md5_append(&state, salt + 3, salt_end - salt - 3);
+        md5_append(&state, (md5_byte_t *)salt + 3, salt_end - salt - 3);
 
       if (i % 7)
-        md5_append(&state, pw, pwlen);
+        md5_append(&state, (md5_byte_t *)pw, pwlen);
 
       if (i & 1)
         md5_append(&state, digest, 16);
       else
-        md5_append(&state, pw, pwlen);
+        md5_append(&state, (md5_byte_t *)pw, pwlen);
 
       md5_finish(&state, digest);
     }
@@ -1335,52 +1560,6 @@ cups_crypt(const char *pw,	/* I - Password string */
 #endif /* !HAVE_LIBPAM */
 
 
-/*
- * 'get_md5_passwd()' - Get an MD5 password.
- */
-
-static char *				/* O - MD5 password string */
-get_md5_passwd(const char *username,	/* I - Username */
-               const char *group,	/* I - Group */
-               char       passwd[33])	/* O - MD5 password string */
-{
-  FILE	*fp;				/* passwd.md5 file */
-  char	filename[1024],			/* passwd.md5 filename */
-	line[256],			/* Line from file */
-	tempuser[33],			/* User from file */
-	tempgroup[33];			/* Group from file */
-
-
-  snprintf(filename, sizeof(filename), "%s/passwd.md5", ServerRoot);
-  if ((fp = fopen(filename, "r")) == NULL)
-    return (NULL);
-
-  while (fgets(line, sizeof(line), fp) != NULL)
-  {
-    if (sscanf(line, "%32[^:]:%32[^:]:%32s", tempuser, tempgroup, passwd) != 3)
-      continue;
-
-    if (strcmp(username, tempuser) == 0 &&
-        (group == NULL || strcmp(group, tempgroup) == 0))
-    {
-     /*
-      * Found the password entry!
-      */
-
-      fclose(fp);
-      return (passwd);
-    }
-  }
-
- /*
-  * Didn't find a password entry - return NULL!
-  */
-
-  fclose(fp);
-  return (NULL);
-}
-
-
 #if HAVE_LIBPAM
 /*
  * 'pam_func()' - PAM conversation function.
@@ -1494,5 +1673,5 @@ to64(char          *s,	/* O - Output string */
 
 
 /*
- * End of "$Id: auth.c,v 1.55 2002/01/02 17:59:13 mike Exp $".
+ * End of "$Id: auth.c,v 1.77 2005/01/03 19:29:59 mike Exp $".
  */
