@@ -1,9 +1,9 @@
 /*
- * "$Id: dirsvc.c,v 1.92 2002/02/12 18:47:13 mike Exp $"
+ * "$Id: dirsvc.c,v 1.140 2005/01/03 19:29:59 mike Exp $"
  *
  *   Directory services routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -87,11 +87,37 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 
   httpSeparate(uri, method, username, host, &port, resource);
 
+ /*
+  * Determine if the URI contains any illegal characters in it...
+  */
+
+  if (strncmp(uri, "ipp://", 6) != 0 ||
+      !host[0] ||
+      (strncmp(resource, "/printers/", 10) != 0 &&
+       strncmp(resource, "/classes/", 9) != 0))
+  {
+    LogMessage(L_ERROR, "ProcessBrowseData: Bad printer URI in browse data: %s",
+               uri);
+    return;
+  }
+
+  if (strchr(resource, '?') != NULL ||
+      (strncmp(resource, "/printers/", 10) == 0 &&
+       strchr(resource + 10, '/') != NULL) ||
+      (strncmp(resource, "/classes/", 9) == 0 &&
+       strchr(resource + 9, '/') != NULL))
+  {
+    LogMessage(L_ERROR, "ProcessBrowseData: Bad resource in browse data: %s",
+               resource);
+    return;
+  }
+    
  /*
   * OK, this isn't a local printer; see if we already have it listed in
   * the Printers list, and add it if not...
   */
 
+  type   |= CUPS_PRINTER_REMOTE;
   update = 0;
   hptr   = strchr(host, '.');
   sptr   = strchr(ServerName, '.');
@@ -129,7 +155,7 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
     {
       if ((p = FindClass(resource + 9)) != NULL)
       {
-        if (strcasecmp(p->hostname, host) != 0 && p->hostname[0])
+        if (p->hostname && strcasecmp(p->hostname, host) != 0)
 	{
 	 /*
 	  * Nope, this isn't the same host; if the hostname isn't the local host,
@@ -139,36 +165,29 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 
 	  if (p->type & CUPS_PRINTER_REMOTE)
 	  {
-	    /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-            strncat(p->name, "@", sizeof(p->name) - 1);
-	    strncat(p->name, p->hostname, sizeof(p->name) - 1);
+            SetStringf(&p->name, "%s@%s", p->name, p->hostname);
 	    SetPrinterAttrs(p);
 	    SortPrinters();
 	  }
 
           p = NULL;
 	}
-	else if (!p->hostname[0])
+	else if (!p->hostname)
 	{
-	  /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-          strncpy(p->hostname, host, sizeof(p->hostname) - 1);
-	  strncpy(p->uri, uri, sizeof(p->uri) - 1);
-	  strncpy(p->device_uri, uri, sizeof(p->device_uri) - 1);
+          SetString(&p->hostname, host);
+	  SetString(&p->uri, uri);
+	  SetString(&p->device_uri, uri);
           update = 1;
         }
       }
       else
-      {
-        strncpy(name, resource + 9, sizeof(name) - 1);
-        name[sizeof(name) - 1] = '\0';
-      }
+        strlcpy(name, resource + 9, sizeof(name));
     }
-    else if (p != NULL && !p->hostname[0])
+    else if (p != NULL && !p->hostname)
     {
-      /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-      strncpy(p->hostname, host, sizeof(p->hostname) - 1);
-      strncpy(p->uri, uri, sizeof(p->uri) - 1);
-      strncpy(p->device_uri, uri, sizeof(p->device_uri) - 1);
+      SetString(&p->hostname, host);
+      SetString(&p->uri, uri);
+      SetString(&p->device_uri, uri);
       update = 1;
     }
 
@@ -186,11 +205,11 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
       * Force the URI to point to the real server...
       */
 
-      p->type = type;
-      /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-      strncpy(p->uri, uri, sizeof(p->uri) - 1);
-      strncpy(p->device_uri, uri, sizeof(p->device_uri) - 1);
-      strncpy(p->hostname, host, sizeof(p->hostname) - 1);
+      p->type      = type & ~CUPS_PRINTER_REJECTING;
+      p->accepting = 1;
+      SetString(&p->uri, uri);
+      SetString(&p->device_uri, uri);
+      SetString(&p->hostname, host);
 
       update = 1;
     }
@@ -210,7 +229,7 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
     {
       if ((p = FindPrinter(resource + 10)) != NULL)
       {
-        if (strcasecmp(p->hostname, host) != 0 && p->hostname[0])
+        if (p->hostname && strcasecmp(p->hostname, host) != 0)
 	{
 	 /*
 	  * Nope, this isn't the same host; if the hostname isn't the local host,
@@ -220,36 +239,29 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 
 	  if (p->type & CUPS_PRINTER_REMOTE)
 	  {
-	    /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-            strncat(p->name, "@", sizeof(p->name) - 1);
-	    strncat(p->name, p->hostname, sizeof(p->name) - 1);
+	    SetStringf(&p->name, "%s@%s", p->name, p->hostname);
 	    SetPrinterAttrs(p);
 	    SortPrinters();
 	  }
 
           p = NULL;
 	}
-	else if (!p->hostname[0])
+	else if (!p->hostname)
 	{
-	  /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-          strncpy(p->hostname, host, sizeof(p->hostname) - 1);
-	  strncpy(p->uri, uri, sizeof(p->uri) - 1);
-	  strncpy(p->device_uri, uri, sizeof(p->device_uri) - 1);
+          SetString(&p->hostname, host);
+	  SetString(&p->uri, uri);
+	  SetString(&p->device_uri, uri);
           update = 1;
         }
       }
       else
-      {
-        strncpy(name, resource + 10, sizeof(name) - 1);
-        name[sizeof(name) - 1] = '\0';
-      }
+        strlcpy(name, resource + 10, sizeof(name));
     }
-    else if (p != NULL && !p->hostname[0])
+    else if (p != NULL && !p->hostname)
     {
-      /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-      strncpy(p->hostname, host, sizeof(p->hostname) - 1);
-      strncpy(p->uri, uri, sizeof(p->uri) - 1);
-      strncpy(p->device_uri, uri, sizeof(p->device_uri) - 1);
+      SetString(&p->hostname, host);
+      SetString(&p->uri, uri);
+      SetString(&p->device_uri, uri);
       update = 1;
     }
 
@@ -267,11 +279,11 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
       * Force the URI to point to the real server...
       */
 
-      p->type = type;
-      /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-      strncpy(p->hostname, host, sizeof(p->hostname) - 1);
-      strncpy(p->uri, uri, sizeof(p->uri) - 1);
-      strncpy(p->device_uri, uri, sizeof(p->device_uri) - 1);
+      p->type      = type & ~CUPS_PRINTER_REJECTING;
+      p->accepting = 1;
+      SetString(&p->hostname, host);
+      SetString(&p->uri, uri);
+      SetString(&p->device_uri, uri);
 
       update = 1;
     }
@@ -282,30 +294,43 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
   */
 
   p->state       = state;
-  p->accepting   = state != IPP_PRINTER_STOPPED;
   p->browse_time = time(NULL);
 
+  if (type & CUPS_PRINTER_REJECTING)
+  {
+    type &= ~CUPS_PRINTER_REJECTING;
+
+    if (p->accepting)
+    {
+      update       = 1;
+      p->accepting = 0;
+    }
+  }
+  else if (!p->accepting)
+  {
+    update       = 1;
+    p->accepting = 1;
+  }
+
   if (p->type != type)
   {
     p->type = type;
     update  = 1;
   }
 
-  if (strcmp(p->location, location))
+  if (location && (!p->location || strcmp(p->location, location)))
   {
-    /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-    strncpy(p->location, location, sizeof(p->location) - 1);
+    SetString(&p->location, location);
     update = 1;
   }
 
-  if (strcmp(p->info, info))
+  if (info && (!p->info || strcmp(p->info, info)))
   {
-    /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-    strncpy(p->info, info, sizeof(p->info) - 1);
+    SetString(&p->info, info);
     update = 1;
   }
 
-  if (!make_model[0])
+  if (!make_model || !make_model[0])
   {
     if (type & CUPS_PRINTER_CLASS)
       snprintf(local_make_model, sizeof(local_make_model),
@@ -318,15 +343,17 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
     snprintf(local_make_model, sizeof(local_make_model),
              "%s on %s", make_model, host);
 
-  if (strcmp(p->make_model, local_make_model))
+  if (!p->make_model || strcmp(p->make_model, local_make_model))
   {
-    /* No "p->var[sizeof(p->var) - 1] = '\0';" because p is zeroed */
-    strncpy(p->make_model, local_make_model, sizeof(p->make_model) - 1);
+    SetString(&p->make_model, local_make_model);
     update = 1;
   }
 
   if (update)
+  {
     SetPrinterAttrs(p);
+    UpdateImplicitClasses();
+  }
 
  /*
   * See if we have a default printer...  If not, make the first printer the
@@ -334,8 +361,12 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
   */
 
   if (DefaultPrinter == NULL && Printers != NULL)
+  {
     DefaultPrinter = Printers;
 
+    WritePrintcap();
+  }
+
  /*
   * Do auto-classing if needed...
   */
@@ -357,10 +388,10 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
       next = p->next;
 
      /*
-      * Skip classes...
+      * Skip implicit classes...
       */
 
-      if (p->type & (CUPS_PRINTER_IMPLICIT | CUPS_PRINTER_CLASS))
+      if (p->type & CUPS_PRINTER_IMPLICIT)
       {
         len = 0;
         continue;
@@ -380,7 +411,7 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 	* we have a class, and if this printer is a member...
 	*/
 
-        if ((pclass = FindPrinter(name)) == NULL)
+        if ((pclass = FindDest(name)) == NULL)
 	{
 	 /*
 	  * Need to add the class...
@@ -391,6 +422,9 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 	  pclass->accepting = 1;
 	  pclass->state     = IPP_PRINTER_IDLE;
 
+          SetString(&pclass->location, p->location);
+          SetString(&pclass->info, p->info);
+
           SetPrinterAttrs(pclass);
 
           LogMessage(L_INFO, "Added implicit class \"%s\"...", name);
@@ -431,7 +465,7 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 	name[len] = '\0';
 	offset    = 0;
 
-	if ((pclass = FindPrinter(name)) != NULL &&
+	if ((pclass = FindDest(name)) != NULL &&
 	    !(pclass->type & CUPS_PRINTER_IMPLICIT))
 	{
 	 /*
@@ -440,7 +474,7 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 	  * the "ImplicitAnyClasses"...
 	  */
 
-          if (ImplicitAnyClasses)
+          if (ImplicitAnyClasses && len < (sizeof(name) - 4))
 	  {
 	   /*
 	    * Add "Any" to the class name...
@@ -477,13 +511,14 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 void
 SendBrowseList(void)
 {
-  printer_t		*p,	/* Current printer */
-			*np;	/* Next printer */
-  time_t		ut,	/* Minimum update time */
-			to;	/* Timeout time */
+  int			count;		/* Number of dests to update */
+  printer_t		*p,		/* Current printer */
+			*np;		/* Next printer */
+  time_t		ut,		/* Minimum update time */
+			to;		/* Timeout time */
 
 
-  if (!Browsing || !(BrowseProtocols & BROWSE_CUPS))
+  if (!Browsing || !BrowseProtocols)
     return;
 
  /*
@@ -493,44 +528,97 @@ SendBrowseList(void)
   ut = time(NULL) - BrowseInterval;
   to = time(NULL) - BrowseTimeout;
 
+ /*
+  * Figure out how many printers need an update...
+  */
+
+  if (BrowseInterval > 0)
+  {
+    int	max_count;			/* Maximum number to update */
+
+
+   /*
+    * Throttle the number of printers we'll be updating this time
+    * around based on the number of queues that need updating and
+    * the maximum number of queues to update each second...
+    */
+
+    max_count = 2 * NumPrinters / BrowseInterval + 1;
+
+    for (count = 0, p = Printers; count < max_count && p != NULL; p = p->next)
+      if (!(p->type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT)) &&
+          p->browse_time < ut)
+        count ++;
+
+   /*
+    * Loop through all of the printers and send local updates as needed...
+    */
+
+    for (p = BrowseNext; count > 0; p = p->next)
+    {
+     /*
+      * Check for wraparound...
+      */
+
+      if (!p)
+        p = Printers;
+
+      if (!p)
+        break;
+      else if (p->type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT))
+        continue;
+      else if (p->browse_time < ut)
+      {
+       /*
+	* Need to send an update...
+	*/
+
+	count --;
+
+	p->browse_time = time(NULL);
+
+	if (BrowseProtocols & BROWSE_CUPS)
+          SendCUPSBrowse(p);
+
+#ifdef HAVE_LIBSLP
+	if (BrowseProtocols & BROWSE_SLP)
+          SendSLPBrowse(p);
+#endif /* HAVE_LIBSLP */
+      }
+    }
+
+   /*
+    * Save where we left off so that all printers get updated...
+    */
+
+    BrowseNext = p;
+  }
+
  /*
   * Loop through all of the printers and send local updates as needed...
   */
 
   for (p = Printers; p != NULL; p = np)
   {
+   /*
+    * Save the next printer pointer...
+    */
+
     np = p->next;
 
+   /*
+    * If this is a remote queue, see if it needs to be timed out...
+    */
+
     if (p->type & CUPS_PRINTER_REMOTE)
     {
-     /*
-      * See if this printer needs to be timed out...
-      */
-
       if (p->browse_time < to)
       {
         LogMessage(L_INFO, "Remote destination \"%s\" has timed out; deleting it...",
 	           p->name);
-        DeletePrinter(p);
+        DeletePrinter(p, 1);
       }
     }
-    else if (p->browse_time < ut &&  BrowseInterval > 0 &&
-            !(p->type & CUPS_PRINTER_IMPLICIT))
-    {
-     /*
-      * Need to send an update...
-      */
-
-      p->browse_time = time(NULL);
-
-      if (BrowseProtocols & BROWSE_CUPS)
-        SendCUPSBrowse(p);
-
-#ifdef HAVE_LIBSLP
-      if (BrowseProtocols & BROWSE_SLP)
-        SendSLPBrowse(p);
-#endif /* HAVE_LIBSLP */
-    }
   }
 }
 
@@ -540,36 +628,134 @@ SendBrowseList(void)
  */
 
 void
-SendCUPSBrowse(printer_t *p)	/* I - Printer to send */
+SendCUPSBrowse(printer_t *p)		/* I - Printer to send */
 {
-  int			i;	/* Looping var */
-  int			bytes;	/* Length of packet */
-  char			packet[1453];
-				/* Browse data packet */
+  int			i;		/* Looping var */
+  cups_ptype_t		type;		/* Printer type */
+  dirsvc_addr_t		*b;		/* Browse address */
+  int			bytes;		/* Length of packet */
+  char			packet[1453];	/* Browse data packet */
+  cups_netif_t		*iface;		/* Network interface */
 
 
-  snprintf(packet, sizeof(packet), "%x %x %s \"%s\" \"%s\" \"%s\"\n",
-           p->type | CUPS_PRINTER_REMOTE, p->state, p->uri,
-	   p->location, p->info, p->make_model);
+ /*
+  * Figure out the printer type value...
+  */
 
-  bytes = strlen(packet);
-  LogMessage(L_DEBUG2, "SendBrowseList: (%d bytes) %s", bytes, packet);
+  type = p->type | CUPS_PRINTER_REMOTE;
+
+  if (!p->accepting)
+    type |= CUPS_PRINTER_REJECTING;
 
  /*
   * Send a packet to each browse address...
   */
 
-  for (i = 0; i < NumBrowsers; i ++)
-    if (sendto(BrowseSocket, packet, bytes, 0,
-	       (struct sockaddr *)Browsers + i, sizeof(Browsers[0])) <= 0)
+  for (i = NumBrowsers, b = Browsers; i > 0; i --, b ++)
+    if (b->iface[0])
     {
-      LogMessage(L_ERROR, "SendBrowseList: sendto failed for browser %d - %s.",
-	         i + 1, strerror(errno));
-      LogMessage(L_ERROR, "Browsing turned off.");
+     /*
+      * Send the browse packet to one or more interfaces...
+      */
 
-      StopBrowsing();
-      Browsing = 0;
-      return;
+      if (strcmp(b->iface, "*") == 0)
+      {
+       /*
+        * Send to all local interfaces...
+	*/
+
+        NetIFUpdate();
+
+        for (iface = NetIFList; iface != NULL; iface = iface->next)
+	{
+	 /*
+	  * Only send to local interfaces...
+	  */
+
+	  if (!iface->is_local || !iface->port)
+	    continue;
+
+	  snprintf(packet, sizeof(packet), "%x %x ipp://%s:%d/%s/%s \"%s\" \"%s\" \"%s\"\n",
+        	   type, p->state, iface->hostname, iface->port,
+		   (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers",
+		   p->name, p->location ? p->location : "",
+		   p->info ? p->info : "",
+		   p->make_model ? p->make_model : "Unknown");
+
+	  bytes = strlen(packet);
+
+	  LogMessage(L_DEBUG2, "SendBrowseList: (%d bytes to \"%s\") %s", bytes,
+        	     iface->name, packet);
+
+          iface->broadcast.sin_port = htons(BrowsePort);
+
+	  sendto(BrowseSocket, packet, bytes, 0,
+		 (struct sockaddr *)&(iface->broadcast),
+		 sizeof(struct sockaddr_in));
+        }
+      }
+      else if ((iface = NetIFFind(b->iface)) != NULL)
+      {
+       /*
+        * Send to the named interface...
+	*/
+
+	if (!iface->port)
+	  continue;
+
+	snprintf(packet, sizeof(packet), "%x %x ipp://%s:%d/%s/%s \"%s\" \"%s\" \"%s\"\n",
+        	 type, p->state, iface->hostname, iface->port,
+		 (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers",
+		 p->name, p->location ? p->location : "",
+		 p->info ? p->info : "",
+		 p->make_model ? p->make_model : "Unknown");
+
+	bytes = strlen(packet);
+
+	LogMessage(L_DEBUG2, "SendBrowseList: (%d bytes to \"%s\") %s", bytes,
+        	   iface->name, packet);
+
+        iface->broadcast.sin_port = htons(BrowsePort);
+
+	sendto(BrowseSocket, packet, bytes, 0,
+	       (struct sockaddr *)&(iface->broadcast),
+	       sizeof(struct sockaddr_in));
+      }
+    }
+    else
+    {
+     /*
+      * Send the browse packet to the indicated address using
+      * the default server name...
+      */
+
+      snprintf(packet, sizeof(packet), "%x %x %s \"%s\" \"%s\" \"%s\"\n",
+       	       type, p->state, p->uri,
+	       p->location ? p->location : "",
+	       p->info ? p->info : "",
+	       p->make_model ? p->make_model : "Unknown");
+
+      bytes = strlen(packet);
+      LogMessage(L_DEBUG2, "SendBrowseList: (%d bytes to %x) %s", bytes,
+        	 (unsigned)ntohl(b->to.sin_addr.s_addr), packet);
+
+      if (sendto(BrowseSocket, packet, bytes, 0,
+		 (struct sockaddr *)&(b->to), sizeof(struct sockaddr_in)) <= 0)
+      {
+       /*
+        * Unable to send browse packet, so remove this address from the
+	* list...
+	*/
+
+	LogMessage(L_ERROR, "SendBrowseList: sendto failed for browser %d - %s.",
+	           b - Browsers + 1, strerror(errno));
+
+        if (i > 1)
+	  memcpy(b, b + 1, (i - 1) * sizeof(dirsvc_addr_t));
+
+	b --;
+	NumBrowsers --;
+      }
     }
 }
 
@@ -585,7 +771,7 @@ StartBrowsing(void)
   struct sockaddr_in	addr;	/* Broadcast address */
 
 
-  if (!Browsing)
+  if (!Browsing || !BrowseProtocols)
     return;
 
   if (BrowseProtocols & BROWSE_CUPS)
@@ -598,7 +784,7 @@ StartBrowsing(void)
     {
       LogMessage(L_ERROR, "StartBrowsing: Unable to create broadcast socket - %s.",
         	 strerror(errno));
-      Browsing = 0;
+      BrowseProtocols &= ~BROWSE_CUPS;
       return;
     }
 
@@ -618,8 +804,8 @@ StartBrowsing(void)
       close(BrowseSocket);
 #endif /* WIN32 */
 
-      BrowseSocket = -1;
-      Browsing     = 0;
+      BrowseSocket    = -1;
+      BrowseProtocols &= ~BROWSE_CUPS;
       return;
     }
 
@@ -643,11 +829,17 @@ StartBrowsing(void)
       close(BrowseSocket);
 #endif /* WIN32 */
 
-      BrowseSocket = -1;
-      Browsing     = 0;
+      BrowseSocket    = -1;
+      BrowseProtocols &= ~BROWSE_CUPS;
       return;
     }
 
+   /*
+    * Close the socket on exec...
+    */
+
+    fcntl(BrowseSocket, F_SETFD, fcntl(BrowseSocket, F_GETFD) | FD_CLOEXEC);
+
    /*
     * Finally, add the socket to the input selection set...
     */
@@ -655,8 +847,10 @@ StartBrowsing(void)
     LogMessage(L_DEBUG2, "StartBrowsing: Adding fd %d to InputSet...",
                BrowseSocket);
 
-    FD_SET(BrowseSocket, &InputSet);
+    FD_SET(BrowseSocket, InputSet);
   }
+  else
+    BrowseSocket = -1;
 
 #ifdef HAVE_LIBSLP
   if (BrowseProtocols & BROWSE_SLP)
@@ -691,7 +885,9 @@ StartPolling(void)
   char		bport[10];	/* Browser port */
   char		interval[10];	/* Poll interval */
   int		statusfds[2];	/* Status pipe */
-  int		fd;		/* Current file descriptor */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  struct sigaction action;	/* POSIX signal handler */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
  /*
@@ -720,7 +916,7 @@ StartPolling(void)
   * polling daemon...
   */
 
-  if (pipe(statusfds))
+  if (cupsdOpenPipe(statusfds))
   {
     LogMessage(L_ERROR, "Unable to create polling status pipes - %s.",
 	       strerror(errno));
@@ -738,6 +934,12 @@ StartPolling(void)
   {
     sprintf(sport, "%d", poll->port);
 
+   /*
+    * Block signals before forking...
+    */
+
+    HoldSignals();
+
     if ((pid = fork()) == 0)
     {
      /*
@@ -753,15 +955,20 @@ StartPolling(void)
 	if (setgid(Group))
           exit(errno);
 
+	if (setgroups(1, &Group))
+          exit(errno);
+
 	if (setuid(User))
           exit(errno);
       }
+      else
+      {
+       /*
+	* Reset group membership to just the main one we belong to.
+	*/
 
-     /*
-      * Reset group membership to just the main one we belong to.
-      */
-
-      setgroups(0, NULL);
+	setgroups(1, &Group);
+      }
 
      /*
       * Redirect stdin and stdout to /dev/null, and stderr to the
@@ -777,8 +984,27 @@ StartPolling(void)
       close(2);
       dup(statusfds[1]);
 
-      for (fd = 3; fd < MaxFDs; fd ++)
-	close(fd);
+     /*
+      * Unblock signals before doing the exec...
+      */
+
+#ifdef HAVE_SIGSET
+      sigset(SIGTERM, SIG_DFL);
+      sigset(SIGCHLD, SIG_DFL);
+#elif defined(HAVE_SIGACTION)
+      memset(&action, 0, sizeof(action));
+
+      sigemptyset(&action.sa_mask);
+      action.sa_handler = SIG_DFL;
+
+      sigaction(SIGTERM, &action, NULL);
+      sigaction(SIGCHLD, &action, NULL);
+#else
+      signal(SIGTERM, SIG_DFL);
+      signal(SIGCHLD, SIG_DFL);
+#endif /* HAVE_SIGSET */
+
+      ReleaseSignals();
 
      /*
       * Execute the polling daemon...
@@ -801,6 +1027,8 @@ StartPolling(void)
       LogMessage(L_DEBUG, "StartPolling: Started polling daemon for %s:%d, pid = %d",
                  poll->hostname, poll->port, pid);
     }
+
+    ReleaseSignals();
   }
 
   close(statusfds[1]);
@@ -812,7 +1040,7 @@ StartPolling(void)
   LogMessage(L_DEBUG2, "StartPolling: Adding fd %d to InputSet...",
              PollPipe);
 
-  FD_SET(PollPipe, &InputSet);
+  FD_SET(PollPipe, InputSet);
 }
 
 
@@ -823,7 +1051,7 @@ StartPolling(void)
 void
 StopBrowsing(void)
 {
-  if (!Browsing)
+  if (!Browsing || !BrowseProtocols)
     return;
 
   if (BrowseProtocols & BROWSE_CUPS)
@@ -843,8 +1071,8 @@ StopBrowsing(void)
       LogMessage(L_DEBUG2, "StopBrowsing: Removing fd %d from InputSet...",
         	 BrowseSocket);
 
-      FD_CLR(BrowseSocket, &InputSet);
-      BrowseSocket = 0;
+      FD_CLR(BrowseSocket, InputSet);
+      BrowseSocket = -1;
     }
   }
 
@@ -878,7 +1106,7 @@ StopPolling(void)
 
     LogMessage(L_DEBUG2, "StopPolling: removing fd %d from InputSet.",
                PollPipe);
-    FD_CLR(PollPipe, &InputSet);
+    FD_CLR(PollPipe, InputSet);
 
     PollPipe = -1;
   }
@@ -900,14 +1128,14 @@ UpdateCUPSBrowse(void)
   int		auth;			/* Authorization status */
   int		len;			/* Length of name string */
   int		bytes;			/* Number of bytes left */
-  char		packet[1540],		/* Broadcast packet */
+  char		packet[1541],		/* Broadcast packet */
 		*pptr;			/* Pointer into packet */
   struct sockaddr_in srcaddr;		/* Source address */
   char		srcname[1024];		/* Source hostname */
   unsigned	address;		/* Source address (host order) */
   struct hostent *srchost;		/* Host entry for source address */
-  cups_ptype_t	type;			/* Printer type */
-  ipp_pstate_t	state;			/* Printer state */
+  unsigned	type;			/* Printer type */
+  unsigned	state;			/* Printer state */
   char		uri[HTTP_MAX_URI],	/* Printer URI */
 		method[HTTP_MAX_URI],	/* Method portion of URI */
 		username[HTTP_MAX_URI],	/* Username portion of URI */
@@ -917,6 +1145,7 @@ UpdateCUPSBrowse(void)
 		location[IPP_MAX_NAME],	/* Location string */
 		make_model[IPP_MAX_NAME];/* Make and model string */
   int		port;			/* Port portion of URI */
+  cups_netif_t	*iface;			/* Network interface */
 
 
  /*
@@ -924,8 +1153,8 @@ UpdateCUPSBrowse(void)
   */
 
   len = sizeof(srcaddr);
-  if ((bytes = recvfrom(BrowseSocket, packet, sizeof(packet), 0, 
-                        (struct sockaddr *)&srcaddr, &len)) <= 0)
+  if ((bytes = recvfrom(BrowseSocket, packet, sizeof(packet) - 1, 0, 
+                        (struct sockaddr *)&srcaddr, &len)) < 0)
   {
    /*
     * "Connection refused" is returned under Linux if the destination port
@@ -933,7 +1162,7 @@ UpdateCUPSBrowse(void)
     * error here and ignore it for now...
     */
 
-    if (errno != ECONNREFUSED)
+    if (errno != ECONNREFUSED && errno != EAGAIN)
     {
       LogMessage(L_ERROR, "Browse recv failed - %s.", strerror(errno));
       LogMessage(L_ERROR, "Browsing turned off.");
@@ -968,10 +1197,7 @@ UpdateCUPSBrowse(void)
     sprintf(srcname, "%d.%d.%d.%d", address >> 24, (address >> 16) & 255,
             (address >> 8) & 255, address & 255);
   else
-  {
-    strncpy(srcname, srchost->h_name, sizeof(srcname) - 1);
-    srcname[sizeof(srcname) - 1] = '\0';
-  }
+    strlcpy(srcname, srchost->h_name, sizeof(srcname));
 
   len = strlen(srcname);
 
@@ -1032,22 +1258,21 @@ UpdateCUPSBrowse(void)
 
   if (auth == AUTH_DENY)
   {
-    LogMessage(L_DEBUG, "UpdateBrowseList: Refused %d bytes from %s", bytes,
+    LogMessage(L_DEBUG, "UpdateCUPSBrowse: Refused %d bytes from %s", bytes,
                srcname);
     return;
   }
 
-  LogMessage(L_DEBUG2, "UpdateBrowseList: (%d bytes from %s) %s", bytes, srcname,
+  LogMessage(L_DEBUG2, "UpdateCUPSBrowse: (%d bytes from %s) %s", bytes, srcname,
              packet);
 
  /*
   * Parse packet...
   */
 
-  if (sscanf(packet, "%x%x%1023s", (unsigned *)&type, (unsigned *)&state,
-             uri) < 3)
+  if (sscanf(packet, "%x%x%1023s", &type, &state, uri) < 3)
   {
-    LogMessage(L_WARN, "UpdateBrowseList: Garbled browse packet - %s",
+    LogMessage(L_WARN, "UpdateCUPSBrowse: Garbled browse packet - %s",
                packet);
     return;
   }
@@ -1074,7 +1299,7 @@ UpdateCUPSBrowse(void)
     if (*pptr == '\"')
       pptr ++;
 
-    while (*pptr && isspace(*pptr))
+    while (*pptr && isspace(*pptr & 255))
       pptr ++;
 
     if (*pptr == '\"')
@@ -1090,7 +1315,7 @@ UpdateCUPSBrowse(void)
       if (*pptr == '\"')
 	pptr ++;
 
-      while (*pptr && isspace(*pptr))
+      while (*pptr && isspace(*pptr & 255))
 	pptr ++;
 
       if (*pptr == '\"')
@@ -1119,9 +1344,19 @@ UpdateCUPSBrowse(void)
 
   DEBUG_printf(("host=\"%s\", ServerName=\"%s\"\n", host, ServerName));
 
+ /*
+  * Check for packets from the local server...
+  */
+
   if (strcasecmp(host, ServerName) == 0)
     return;
 
+  NetIFUpdate();
+
+  for (iface = NetIFList; iface != NULL; iface = iface->next)
+    if (strcasecmp(host, iface->hostname) == 0)
+      return;
+
  /*
   * Do relaying...
   */
@@ -1132,7 +1367,7 @@ UpdateCUPSBrowse(void)
                  (struct sockaddr *)&(Relays[i].to),
 		 sizeof(struct sockaddr_in)) <= 0)
       {
-	LogMessage(L_ERROR, "UpdateBrowseList: sendto failed for relay %d - %s.",
+	LogMessage(L_ERROR, "UpdateCUPSBrowse: sendto failed for relay %d - %s.",
 	           i + 1, strerror(errno));
 	return;
       }
@@ -1141,7 +1376,8 @@ UpdateCUPSBrowse(void)
   * Process the browse data...
   */
 
-  ProcessBrowseData(uri, type, state, location, info, make_model);
+  ProcessBrowseData(uri, (cups_ptype_t)type, (ipp_pstate_t)state, location,
+                    info, make_model);
 }
 
 
@@ -1184,13 +1420,20 @@ UpdatePolling(void)
 
     *lineptr++ = '\0';
 
-    LogMessage(L_ERROR, "%s", buffer);
+    if (!strncmp(buffer, "ERROR: ", 7))
+      LogMessage(L_ERROR, "%s", buffer + 7);
+    else if (!strncmp(buffer, "DEBUG: ", 7))
+      LogMessage(L_DEBUG, "%s", buffer + 7);
+    else if (!strncmp(buffer, "DEBUG2: ", 8))
+      LogMessage(L_DEBUG2, "%s", buffer + 8);
+    else
+      LogMessage(L_DEBUG, "%s", buffer);
 
    /*
     * Copy over the buffer data we've used up...
     */
 
-    strcpy(buffer, lineptr);
+    cups_strcpy(buffer, lineptr);
     bufused -= lineptr - buffer;
 
     if (bufused < 0)
@@ -1297,7 +1540,7 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   if (p->type & CUPS_PRINTER_BIND)
   {
     if (finishings[0])
-      strncat(finishings, ",bind", sizeof(finishings) - 1);
+      strlcat(finishings, ",bind", sizeof(finishings));
     else
       strcpy(finishings, "bind");
   }
@@ -1305,7 +1548,7 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   if (p->type & CUPS_PRINTER_PUNCH)
   {
     if (finishings[0])
-      strncat(finishings, ",punch", sizeof(finishings) - 1);
+      strlcat(finishings, ",punch", sizeof(finishings));
     else
       strcpy(finishings, "punch");
   }
@@ -1313,7 +1556,7 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   if (p->type & CUPS_PRINTER_COVER)
   {
     if (finishings[0])
-      strncat(finishings, ",cover", sizeof(finishings) - 1);
+      strlcat(finishings, ",cover", sizeof(finishings));
     else
       strcpy(finishings, "cover");
   }
@@ -1321,7 +1564,7 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   if (p->type & CUPS_PRINTER_SORT)
   {
     if (finishings[0])
-      strncat(finishings, ",sort", sizeof(finishings) - 1);
+      strlcat(finishings, ",sort", sizeof(finishings));
     else
       strcpy(finishings, "sort");
   }
@@ -1329,15 +1572,12 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   if (!finishings[0])
     strcpy(finishings, "none");
 
-  finishings[sizeof(finishings) - 1] = '\0';
-
  /*
-  * Quote any commas in the make and model, location, and info strings
-  * (local strings are twice the size of the ones in the printer_t
-  * structure, so no buffer overflow is possible...)
+  * Quote any commas in the make and model, location, and info strings...
   */
 
-  for (src = p->make_model, dst = make_model; *src;)
+  for (src = p->make_model, dst = make_model;
+       src && *src && dst < (make_model + sizeof(make_model) - 2);)
   {
     if (*src == ',' || *src == '\\' || *src == ')')
       *dst++ = '\\';
@@ -1350,7 +1590,8 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   if (!make_model[0])
     strcpy(make_model, "Unknown");
 
-  for (src = p->location, dst = location; *src;)
+  for (src = p->location, dst = location;
+       src && *src && dst < (location + sizeof(location) - 2);)
   {
     if (*src == ',' || *src == '\\' || *src == ')')
       *dst++ = '\\';
@@ -1363,7 +1604,8 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   if (!location[0])
     strcpy(location, "Unknown");
 
-  for (src = p->info, dst = info; *src;)
+  for (src = p->info, dst = info;
+       src && *src && dst < (info + sizeof(info) - 2);)
   {
     if (*src == ',' || *src == '\\' || *src == ')')
       *dst++ = '\\';
@@ -1391,11 +1633,11 @@ SendSLPBrowse(printer_t *p)		/* I - Printer to register */
   snprintf(attrs, sizeof(attrs),
            "(printer-uri-supported=%s),"
            "(uri-authentication-supported=%s>),"
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
            "(uri-security-supported=tls>),"
 #else
            "(uri-security-supported=none>),"
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
            "(printer-name=%s),"
            "(printer-location=%s),"
            "(printer-info=%s),"
@@ -1466,14 +1708,13 @@ SLPDeregPrinter(printer_t *p)
 int 					/* O - 0 on success */
 GetSlpAttrVal(const char *attrlist,	/* I - Attribute list string */
               const char *tag,		/* I - Name of attribute */
-              char       *valbuf,	/* O - Value */
-              int        valbuflen)	/* I - Max length of value */
+              char       **valbuf)	/* O - Value */
 {
   char	*ptr1,				/* Pointer into string */
 	*ptr2;				/* ... */
 
 
-  valbuf[0] = '\0';
+  ClearString(valbuf);
 
   if ((ptr1 = strstr(attrlist, tag)) != NULL)
   {
@@ -1481,25 +1722,22 @@ GetSlpAttrVal(const char *attrlist,	/* I - Attribute list string */
 
     if ((ptr2 = strchr(ptr1,')')) != NULL)
     {
-      if (valbuflen > (ptr2 - ptr1))
-      {
-       /*
-        * Copy the value...
-	*/
+     /*
+      * Copy the value...
+      */
 
-        strncpy(valbuf, ptr1, ptr2 - ptr1);
-	valbuf[ptr2 - ptr1] = '\0';
+      *valbuf = calloc(ptr2 - ptr1 + 1, 1);
+      strncpy(*valbuf, ptr1, ptr2 - ptr1);
 
-       /*
-        * Dequote the value...
-	*/
+     /*
+      * Dequote the value...
+      */
 
-	for (ptr1 = valbuf; *ptr1; ptr1 ++)
-	  if (*ptr1 == '\\' && ptr1[1])
-	    strcpy(ptr1, ptr1 + 1);
+      for (ptr1 = *valbuf; *ptr1; ptr1 ++)
+	if (*ptr1 == '\\' && ptr1[1])
+	  cups_strcpy(ptr1, ptr1 + 1);
 
-        return (0);
-      }
+      return (0);
     }
   }
 
@@ -1511,13 +1749,13 @@ GetSlpAttrVal(const char *attrlist,	/* I - Attribute list string */
  * 'AttrCallback()' - SLP attribute callback 
  */
 
-SLPBoolean
-AttrCallback(SLPHandle  hslp, 
-             const char *attrlist, 
-             SLPError   errcode, 
-             void       *cookie)
+SLPBoolean				/* O - SLP_TRUE for success */
+AttrCallback(SLPHandle  hslp,		/* I - SLP handle */
+             const char *attrlist,	/* I - Attribute list */
+             SLPError   errcode,	/* I - Parsing status for this attr */
+             void       *cookie)	/* I - Current printer */
 {
-  char         tmp[IPP_MAX_NAME];
+  char         *tmp = 0;
   printer_t    *p = (printer_t*)cookie;
 
 
@@ -1542,19 +1780,19 @@ AttrCallback(SLPHandle  hslp,
 
   p->type = CUPS_PRINTER_REMOTE;
 
-  if (GetSlpAttrVal(attrlist, "(printer-location=", p->location,
-                    sizeof(p->location)))
+  if (GetSlpAttrVal(attrlist, "(printer-location=", &(p->location)))
     return (SLP_FALSE);
-  if (GetSlpAttrVal(attrlist, "(printer-make-and-model=", p->make_model,
-                    sizeof(p->make_model)))
+  if (GetSlpAttrVal(attrlist, "(printer-info=", &(p->info)))
+    return (SLP_FALSE);
+  if (GetSlpAttrVal(attrlist, "(printer-make-and-model=", &(p->make_model)))
     return (SLP_FALSE);
 
-  if (GetSlpAttrVal(attrlist, "(color-supported=", tmp, sizeof(tmp)))
+  if (GetSlpAttrVal(attrlist, "(color-supported=", &tmp))
     return (SLP_FALSE);
   if (strcasecmp(tmp, "true") == 0)
     p->type |= CUPS_PRINTER_COLOR;
 
-  if (GetSlpAttrVal(attrlist, "(finishings-supported=", tmp, sizeof(tmp)))
+  if (GetSlpAttrVal(attrlist, "(finishings-supported=", &tmp))
     return (SLP_FALSE);
   if (strstr(tmp, "staple"))
     p->type |= CUPS_PRINTER_STAPLE;
@@ -1563,11 +1801,13 @@ AttrCallback(SLPHandle  hslp,
   if (strstr(tmp, "punch"))
     p->type |= CUPS_PRINTER_PUNCH;
 
-  if (GetSlpAttrVal(attrlist, "(sides-supported=", tmp, sizeof(tmp)))
+  if (GetSlpAttrVal(attrlist, "(sides-supported=", &tmp))
     return (SLP_FALSE);
   if (strstr(tmp,"two-sided"))
     p->type |= CUPS_PRINTER_DUPLEX;
 
+  ClearString(&tmp);
+
   return (SLP_TRUE);
 }
 
@@ -1618,7 +1858,7 @@ SrvUrlCallback(SLPHandle      hslp, 	/* I - SLP handle */
   * Copy the SLP service URL...
   */
 
-  strncpy(s->url, srvurl, sizeof(s->url));
+  strlcpy(s->url, srvurl, sizeof(s->url));
 
  /* 
   * Link the SLP service URL into the head of the list
@@ -1657,7 +1897,7 @@ UpdateSLPBrowse(void)
   * Reset the refresh time...
   */
 
-  BrowseSLPRefresh = time(NULL) + BrowseTimeout - BrowseInterval;
+  BrowseSLPRefresh = time(NULL) + BrowseInterval;
 
  /* 
   * Poll for remote printers using SLP...
@@ -1674,6 +1914,12 @@ UpdateSLPBrowse(void)
 
   for (; s; s = next)
   {
+   /*
+    * Save the "next" pointer...
+    */
+
+    next = s->next;
+
    /* 
     * Load a printer_t structure with the SLP service attributes...
     */
@@ -1712,10 +1958,9 @@ UpdateSLPBrowse(void)
     }
 
    /*
-    * Save the "next" pointer and free this listing...
+    * Free this listing...
     */
 
-    next = s->next;
     free(s);
   }       
 
@@ -1725,5 +1970,5 @@ UpdateSLPBrowse(void)
 
 
 /*
- * End of "$Id: dirsvc.c,v 1.92 2002/02/12 18:47:13 mike Exp $".
+ * End of "$Id: dirsvc.c,v 1.140 2005/01/03 19:29:59 mike Exp $".
  */
