@@ -1,9 +1,9 @@
 /*
- * "$Id: ppds.c,v 1.22 2002/01/02 17:59:17 mike Exp $"
+ * "$Id: ppds.c,v 1.37 2005/01/03 19:29:59 mike Exp $"
  *
  *   PPD scanning routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -30,7 +30,6 @@
  *   compare_names() - Compare PPD filenames for sorting.
  *   compare_ppds()  - Compare PPD file make and model names for sorting.
  *   load_ppds()     - Load PPD files recursively.
- *   ppd_gets()      - Read a line from a PPD file.
  */
 
 /*
@@ -40,15 +39,6 @@
 #include "cupsd.h"
 #include <ctype.h>
 
-#ifdef HAVE_LIBZ
-#  include <zlib.h>
-#else
-#  define gzFile	FILE *
-#  define gzclose	fclose
-#  define gzread(f,b,l)	fread((b), 1, (l), (f))
-#  define gzopen	fopen
-#endif /* HAVE_LIBZ */
-
 
 /*
  * PPD information structures...
@@ -71,24 +61,12 @@ typedef struct
 } ppd_info_t;
 
 
-/*
- * Buffered file structure...
- */
-
-typedef struct
-{
-  gzFile	fp;			/* Pointer to file */
-  char		*ptr,			/* Pointer in buffer */
-		*end,			/* End of buffer */
-		buf[1024];		/* Buffer */
-} buf_t;
-
-
 /*
  * Local globals...
  */
 
 static int		num_ppds,	/* Number of PPD files */
+			sorted_ppds,	/* Number of sorted PPD files */
 			alloc_ppds;	/* Number of allocated entries */
 static ppd_info_t	*ppds;		/* PPD file info */
 static int		changed_ppd;	/* Did we change the PPD database? */
@@ -98,11 +76,9 @@ static int		changed_ppd;	/* Did we change the PPD database? */
  * Local functions...
  */
 
-static int	buf_read(buf_t *fp);
 static int	compare_names(const ppd_info_t *p0, const ppd_info_t *p1);
 static int	compare_ppds(const ppd_info_t *p0, const ppd_info_t *p1);
 static void	load_ppds(const char *d, const char *p);
-static char	*ppd_gets(buf_t *fp, char *buf, int buflen);
 
 
 /*
@@ -110,13 +86,13 @@ static char	*ppd_gets(buf_t *fp, char *buf, int buflen);
  */
 
 void
-LoadPPDs(const char *d)		/* I - Directory to scan... */
+LoadPPDs(const char *d)			/* I - Directory to scan... */
 {
-  int		i;		/* Looping var */
-  ppd_info_t	*ppd;		/* Current PPD file */
-  FILE		*fp;		/* ppds.dat file */
-  struct stat	fileinfo;	/* ppds.dat information */
-  char		filename[1024];	/* ppds.dat filename */
+  int		i;			/* Looping var */
+  ppd_info_t	*ppd;			/* Current PPD file */
+  cups_file_t	*fp;			/* ppds.dat file */
+  struct stat	fileinfo;		/* ppds.dat information */
+  char		filename[1024];		/* ppds.dat filename */
 
 
  /*
@@ -145,15 +121,15 @@ LoadPPDs(const char *d)		/* I - Directory to scan... */
       num_ppds   = 0;
       alloc_ppds = 0;
     }
-    else if ((fp = fopen(filename, "rb")) != NULL)
+    else if ((fp = cupsFileOpen(filename, "rb")) != NULL)
     {
       for (i = num_ppds, ppd = ppds; i > 0; i --, ppd ++)
       {
-        fread(&(ppd->record), 1, sizeof(ppd_rec_t), fp);
+        cupsFileRead(fp, (char *)&(ppd->record), sizeof(ppd_rec_t));
 	ppd->found = 0;
       }
 
-      fclose(fp);
+      cupsFileClose(fp);
 
       LogMessage(L_INFO, "LoadPPDs: Read \"%s\", %d PPDs...", filename,
                  num_ppds);
@@ -163,8 +139,10 @@ LoadPPDs(const char *d)		/* I - Directory to scan... */
       */
 
       if (num_ppds > 1)
+      {
 	qsort(ppds, num_ppds, sizeof(ppd_info_t),
               (int (*)(const void *, const void *))compare_names);
+      }
     }
     else
     {
@@ -178,6 +156,8 @@ LoadPPDs(const char *d)		/* I - Directory to scan... */
   * Load all PPDs in the specified directory and below...
   */
 
+  sorted_ppds = num_ppds;
+
   load_ppds(d, "");
 
  /*
@@ -192,7 +172,7 @@ LoadPPDs(const char *d)		/* I - Directory to scan... */
       */
 
       if (i > 1)
-        memcpy(ppd, ppd + 1, (i - 1) * sizeof(ppd_info_t));
+        memmove(ppd, ppd + 1, (i - 1) * sizeof(ppd_info_t));
 
       num_ppds --;
       ppd --;
@@ -212,12 +192,12 @@ LoadPPDs(const char *d)		/* I - Directory to scan... */
 
   if (changed_ppd)
   {
-    if ((fp = fopen(filename, "wb")) != NULL)
+    if ((fp = cupsFileOpen(filename, "wb")) != NULL)
     {
       for (i = num_ppds, ppd = ppds; i > 0; i --, ppd ++)
-	fwrite(&(ppd->record), 1, sizeof(ppd_rec_t), fp);
+	cupsFileWrite(fp, (char *)&(ppd->record), sizeof(ppd_rec_t));
 
-      fclose(fp);
+      cupsFileClose(fp);
 
       LogMessage(L_INFO, "LoadPPDs: Wrote \"%s\", %d PPDs...", filename,
         	 num_ppds);
@@ -278,31 +258,6 @@ LoadPPDs(const char *d)		/* I - Directory to scan... */
 }
 
 
-/*
- * 'buf_read()' - Read a buffer of data into memory...
- */
-
-static int			/* O - Number of bytes read */
-buf_read(buf_t *fp)		/* I - File to read from */
-{
-  int	count;			/* Number of bytes read */
-
-
-  if ((count = gzread(fp->fp, fp->buf, sizeof(fp->buf))) > 0)
-  {
-    fp->ptr = fp->buf;
-    fp->end = fp->buf + count;
-  }
-  else
-  {
-    fp->ptr = NULL;
-    fp->end = NULL;
-  }
-
-  return (count);
-}
-
-
 /*
  * 'compare_names()' - Compare PPD filenames for sorting.
  */
@@ -350,7 +305,7 @@ compare_ppds(const ppd_info_t *p0,	/* I - First PPD file */
 
   while (*s && *t)
   {
-    if (isdigit(*s) && isdigit(*t))
+    if (isdigit(*s & 255) && isdigit(*t & 255))
     {
      /*
       * Got a number; start by skipping leading 0's...
@@ -365,7 +320,7 @@ compare_ppds(const ppd_info_t *p0,	/* I - First PPD file */
       * Skip equal digits...
       */
 
-      while (isdigit(*s) && *s == *t)
+      while (isdigit(*s & 255) && *s == *t)
       {
         s ++;
 	t ++;
@@ -375,11 +330,11 @@ compare_ppds(const ppd_info_t *p0,	/* I - First PPD file */
       * Bounce out if *s and *t aren't both digits...
       */
 
-      if (isdigit(*s) && !isdigit(*t))
+      if (isdigit(*s & 255) && !isdigit(*t & 255))
         return (1);
-      else if (!isdigit(*s) && isdigit(*t))
+      else if (!isdigit(*s & 255) && isdigit(*t & 255))
         return (-1);
-      else if (!isdigit(*s) || !isdigit(*t))
+      else if (!isdigit(*s & 255) || !isdigit(*t & 255))
         continue;     
 
       if (*s < *t)
@@ -395,13 +350,13 @@ compare_ppds(const ppd_info_t *p0,	/* I - First PPD file */
       s ++;
       t ++;
 
-      while (isdigit(*s))
+      while (isdigit(*s & 255))
       {
         digits ++;
 	s ++;
       }
 
-      while (isdigit(*t))
+      while (isdigit(*t & 255))
       {
         digits --;
 	t ++;
@@ -418,9 +373,9 @@ compare_ppds(const ppd_info_t *p0,	/* I - First PPD file */
       else if (diff)
         return (diff);
     }
-    else if (tolower(*s) < tolower(*t))
+    else if (tolower(*s & 255) < tolower(*t & 255))
       return (-1);
-    else if (tolower(*s) > tolower(*t))
+    else if (tolower(*s & 255) > tolower(*t & 255))
       return (1);
     else
     {
@@ -452,7 +407,7 @@ load_ppds(const char *d,		/* I - Actual directory */
           const char *p)		/* I - Virtual path in name */
 {
   int		i;			/* Looping var */
-  buf_t		fp;			/* Pointer to file */
+  cups_file_t	*fp;			/* Pointer to file */
   DIR		*dir;			/* Directory pointer */
   DIRENT	*dent;			/* Directory entry */
   struct stat	fileinfo;		/* File information */
@@ -476,18 +431,21 @@ load_ppds(const char *d,		/* I - Actual directory */
   }		languages[] =
   {
     { "chinese",	"cn" },
+    { "danish",		"da" },
+    { "dutch",		"nl" },
     { "english",	"en" },
+    { "finnish",	"fi" },
     { "french",		"fr" },
     { "german",		"de" },
-    { "danish",		"da" },
-    { "finnish",	"fi" },
+    { "greek",		"el" },
     { "italian",	"it" },
-    { "dutch",		"du" },
     { "japanese",	"jp" },
     { "norwegian",	"no" },
     { "polish",		"pl" },
-    { "portugese",	"pt" },
+    { "portuguese",	"pt" },
     { "russian",	"ru" },
+    { "slovak",		"sk" },
+    { "spanish",	"es" },
     { "swedish",	"sv" },
     { "turkish",	"tr" }
   };
@@ -518,10 +476,7 @@ load_ppds(const char *d,		/* I - Actual directory */
     if (p[0])
       snprintf(name, sizeof(name), "%s/%s", p, dent->d_name);
     else
-    {
-      strncpy(name, dent->d_name, sizeof(name) - 1);
-      name[sizeof(name) - 1] = '\0';
-    }
+      strlcpy(name, dent->d_name, sizeof(name));
 
     if (stat(filename, &fileinfo))
       continue;
@@ -540,11 +495,11 @@ load_ppds(const char *d,		/* I - Actual directory */
     * See if this file has been scanned before...
     */
 
-    if (num_ppds > 0)
+    if (sorted_ppds > 0)
     {
       strcpy(key.record.ppd_name, name);
 
-      ppd = bsearch(&key, ppds, num_ppds, sizeof(ppd_info_t),
+      ppd = bsearch(&key, ppds, sorted_ppds, sizeof(ppd_info_t),
                     (int (*)(const void *, const void *))compare_names);
 
       if (ppd &&
@@ -562,17 +517,15 @@ load_ppds(const char *d,		/* I - Actual directory */
     * No, file is new/changed, so re-scan it...
     */
 
-    if ((fp.fp = gzopen(filename, "rb")) == NULL)
+    if ((fp = cupsFileOpen(filename, "rb")) == NULL)
       continue;
 
-    fp.ptr = fp.end = NULL;
-
    /*
     * Now see if this is a PPD file...
     */
 
     line[0] = '\0';
-    ppd_gets(&fp, line, sizeof(line));
+    cupsFileGets(fp, line, sizeof(line));
 
     if (strncmp(line, "*PPD-Adobe:", 11) != 0)
     {
@@ -580,7 +533,7 @@ load_ppds(const char *d,		/* I - Actual directory */
       * Nope, close the file and continue...
       */
 
-      gzclose(fp.fp);
+      cupsFileClose(fp);
 
       continue;
     }
@@ -594,7 +547,7 @@ load_ppds(const char *d,		/* I - Actual directory */
     manufacturer[0] = '\0';
     strcpy(language, "en");
 
-    while (ppd_gets(&fp, line, sizeof(line)) != NULL)
+    while (cupsFileGets(fp, line, sizeof(line)) != NULL)
     {
       if (strncmp(line, "*Manufacturer:", 14) == 0)
 	sscanf(line, "%*[^\"]\"%255[^\"]", manufacturer);
@@ -628,7 +581,7 @@ load_ppds(const char *d,		/* I - Actual directory */
     * Close the file...
     */
 
-    gzclose(fp.fp);
+    cupsFileClose(fp);
 
    /*
     * See if we got all of the required info...
@@ -639,8 +592,8 @@ load_ppds(const char *d,		/* I - Actual directory */
     else
       strcpy(make_model, model_name);
 
-    while (isspace(make_model[0]))
-      strcpy(make_model, make_model + 1);
+    while (isspace(make_model[0] & 255))
+      cups_strcpy(make_model, make_model + 1);
 
     if (!make_model[0])
       continue;	/* Nope... */
@@ -649,8 +602,8 @@ load_ppds(const char *d,		/* I - Actual directory */
     * See if we got a manufacturer...
     */
 
-    while (isspace(manufacturer[0]))
-      strcpy(manufacturer, manufacturer + 1);
+    while (isspace(manufacturer[0] & 255))
+      cups_strcpy(manufacturer, manufacturer + 1);
 
     if (!manufacturer[0] || strcmp(manufacturer, "ESP") == 0)
     {
@@ -658,7 +611,7 @@ load_ppds(const char *d,		/* I - Actual directory */
       * Nope, copy the first part of the make and model then...
       */
 
-      strncpy(manufacturer, make_model, sizeof(manufacturer) - 1);
+      strlcpy(manufacturer, make_model, sizeof(manufacturer));
 
      /*
       * Truncate at the first space, dash, or slash, or make the
@@ -721,7 +674,7 @@ load_ppds(const char *d,		/* I - Actual directory */
       */
 
       country[0] = '_';
-      strcpy(country + 1, ptr);
+      cups_strcpy(country + 1, ptr);
     }
     else
     {
@@ -807,24 +760,16 @@ load_ppds(const char *d,		/* I - Actual directory */
     ppd->record.ppd_mtime = fileinfo.st_mtime;
     ppd->record.ppd_size  = fileinfo.st_size;
 
-    strncpy(ppd->record.ppd_name, name,
-            sizeof(ppd->record.ppd_name) - 1);
-    strncpy(ppd->record.ppd_make, manufacturer,
-            sizeof(ppd->record.ppd_make) - 1);
-    strncpy(ppd->record.ppd_make_and_model, make_model,
-            sizeof(ppd->record.ppd_make_and_model) - 1);
-    strncpy(ppd->record.ppd_natural_language, language,
-            sizeof(ppd->record.ppd_natural_language) - 1);
+    strlcpy(ppd->record.ppd_name, name,
+            sizeof(ppd->record.ppd_name));
+    strlcpy(ppd->record.ppd_make, manufacturer,
+            sizeof(ppd->record.ppd_make));
+    strlcpy(ppd->record.ppd_make_and_model, make_model,
+            sizeof(ppd->record.ppd_make_and_model));
+    strlcpy(ppd->record.ppd_natural_language, language,
+            sizeof(ppd->record.ppd_natural_language));
 
     changed_ppd = 1;
-
-   /*
-    * Re-sort the PPD array...
-    */
-
-    if (num_ppds > 1 && new_ppd)
-      qsort(ppds, num_ppds, sizeof(ppd_info_t),
-            (int (*)(const void *, const void *))compare_names);
   }
 
   closedir(dir);
@@ -832,66 +777,5 @@ load_ppds(const char *d,		/* I - Actual directory */
 
 
 /*
- * 'ppd_gets()' - Read a line from a PPD file.
- */
-
-static char *			/* O - Line from file or NULL on EOF */
-ppd_gets(buf_t *fp,		/* I - File to read from */
-         char  *buf,		/* I - Line buffer */
-	 int   buflen)		/* I - Length of buffer */
-{
-  int		ch;		/* Character from file */
-  char		*ptr,		/* Current position in line buffer */
-		*end;		/* End of line buffer */
-
- /*
-  * Range check everything...
-  */
-
-  if (fp == NULL || buf == NULL || buflen < 2)
-    return (NULL);
-
- /*
-  * Now loop until we have a valid line...
-  */
-
-  ptr = buf;
-  end = buf + buflen - 1;
-
-  for (;;)
-  {
-    if (fp->ptr >= fp->end)
-      if (buf_read(fp) <= 0)
-        break;
-
-    ch = *(fp->ptr)++;
-
-    if (ch == '\r' || ch == '\n')
-    {
-     /*
-      * Line feed or carriage return...
-      */
-
-      if (ptr == buf)		/* Skip blank lines */
-        continue;
-
-      break;
-    }
-    else if (ptr < end)
-      *ptr++ = ch;
-  }
-
-  if (ptr > buf)
-  {
-    *ptr = '\0';
-
-    return (buf);
-  }
-  else
-    return (NULL);
-}
-
-
-/*
- * End of "$Id: ppds.c,v 1.22 2002/01/02 17:59:17 mike Exp $".
+ * End of "$Id: ppds.c,v 1.37 2005/01/03 19:29:59 mike Exp $".
  */
